function sip_solve(stub)
%   sip_solve solves a semi-infinite programming problem and
% print some information in a result file.
%
% In arguments:
%   stub:  stub name of problem
%
% Out arguments:
%   None
%
%
%  27/11/2002 aivaz@dps.uminho.pt

if nargin < 1 | nargout > 0
    error('Invalid number of arguments');
end;

ResultFile = 'results';

% initialise the problem data
[x0, xbl, xbu, tbl, tbu] = sip_init(stub);

% check for no initial guess
if isnan(x0)
    x0=rand(1,length(xbl));
end

% we need an initial guess for the infinite constraints
NumberInfVars = length(tbl);
t0 =(tbu-tbl)/2;

% one call to sip_conval to know how many infinite constraints there are
[c, ceq, inf] = sip_conval(x0, t0); 

NumberInfCons = length(inf);

% solve the problem
[x, fval, exitflag, output] = fseminf('sip_fun', x0, NumberInfCons, 'sip_con',...
    [], [], [], [], xbl, xbu, [], tbl, tbu);

% write the results
fid = fopen(ResultFile,'a');
fprintf(fid,'%s & %d & %d & %f\\\\\n', stub, output.iterations,...
    output.funcCount, fval);
fclose(fid);

disp('Solution:');
disp(x);
disp('Objective value:');
disp(fval);
