#include "dual.h"
#include "int.h"

/* External data, for dual_objfun */

double *dual_h;
double *dual_x;
double *dual_xgrd;
int     dual_n;
double *dual_sol;
double  dual_posdef;

#ifdef NPSOL

/* It could be more specific about NPSOL, in just removing the
   interface to npsol_.
*/

void dual_lags()
{
  double dummy, f0, nor1, nor2;
  double den, pterm;
  double miu, dual_prec, step;
  int ko, i, k;

  FILE *fp;


#if defined __linux__ || defined linux
  double tsample[ntsip];
  double dir0[nxsip], dir1[nxsip];
  double x0[nxsip], x1[nxsip], g1[nxsip], g0[nxsip];
  double lg1[nxsip], lg0[nxsip];
  double gmf[nxsip], *ini_sol[nxsip]; /* nxsip-1 */
#else
  double *tsample, *dir0, *dir1, *x0, *x1, *g1, *g0, *lg1, *lg0;
  double *gmf, *(*ini_sol);
  tsample=mymalloc(ntsip*sizeof(double));
  dir0=mymalloc(nxsip*sizeof(double));
  dir1=mymalloc(nxsip*sizeof(double));
  x0=mymalloc(nxsip*sizeof(double));
  x1=mymalloc(nxsip*sizeof(double));
  g1=mymalloc(nxsip*sizeof(double));
  g0=mymalloc(nxsip*sizeof(double));
  lg1=mymalloc(nxsip*sizeof(double));
  lg0=mymalloc(nxsip*sizeof(double));
  gmf=mymalloc(nxsip*sizeof(double));
  ini_sol=mymalloc((nxsip-1)*sizeof(double *));
#endif



  dual_h=mymalloc(nxsip*nxsip*sizeof(double));
  dual_x=x0; /* data for dual_objfun */
  dual_xgrd=g0;

  /* Outer iteration */
  
  memcpy(x1, X, nxsip*sizeof(double)); /* x1 with initial guess */
  dummy=eval_sip(SIP_OBJGRD, 0, x1, NULL, g1, NULL, NULL, NULL);


  /* init solution initial guesses */
  for(i=0;i<nxsip;i++){
    dual_n=i+1;
    dual_sol=ini_sol[i]=mymalloc((dual_n+ntsipc*(2*dual_n+1))
				 *sizeof(double));
    if(!Sip_Opt.dual_ini_res){
      step=(TBU[0]-TBL[0])/(dual_n+1);
      for(k=0;k<dual_n;k++)
	dual_sol[k]=TBL[0]+step*(k+1);
      for(k=dual_n;k<dual_n*(ntsipc+1);k++)
	dual_sol[k]=1.0;
      for(k=dual_n*(ntsipc+1);k<dual_n+ntsipc*dual_n+ntsipc*(dual_n+1);k++)
	dual_sol[k]=1.0;
    }
  }
  
    
  /*    for(i=0;i<nxsip;i++){ */
  /*      dual_sol=ini_sol[i]; */
  /*      dual_n=i+1; */
  /*      printf("Dual guess with dual_n=%d npsol_n=%d\n", dual_n,
	  dual_n+ntsipc*dual_n+ntsipc*(dual_n+1)); */
  /*      printarray(dual_sol, dual_n+ntsipc*dual_n+ntsipc*(dual_n+1)); */
  /*    } */

  dual_prec=1e-1;
  dual_sol=ini_sol[0];
  dual_n=1;


  miu=0;

  for(ko=0;ko<Sip_Opt.maxitero;ko++){

    set_h(dual_h, ko, nxsip, x1, x0, lg1, lg0);

    /*      printf("Hessian inverse\n"); */
    /*      printmatrix(dual_h, nxsip, nxsip); */


    /* x0 with initial guess for inner iter */
    memcpy(x0, x1, nxsip*sizeof(double));
    memcpy(g0, g1, nxsip*sizeof(double));
    memcpy(lg0, lg1, nxsip*sizeof(double));

    

    for(k=0;;k++,dual_n++){

      memcpy(dir0, dir1, nxsip*sizeof(double));
      
      dual_sol=ini_sol[dual_n-1];
      printf("In get_dir\n");
      get_dir_dualQP(dir1);
      printf("Out get_dir\n");
      
      printf("Subdir\n");
      printarray(dir1, nxsip);
      if(k>0){
	nor1=0.0;
	nor2=0.0;
	for(i=0;i<nxsip;i++){
	  nor1+=pow(dir1[i]-dir0[i], 2.0);
	  nor2+=pow(dir1[i], 2.0);
	}
	nor1=sqrt(nor1);
	nor2=sqrt(nor2);
	if(nor2<Sip_Opt.zero){
	  den=1.0;
	} else {
	  den=nor2;
	}
  	printf("Norm Subdir %f\n", nor1/den);
	printf("norma da direccao %f\n", nor2);
	
	
	if((nor1/den)<dual_prec || dual_n>=nxsip)
	  break;
      } else {
  	if(dual_n>=nxsip)
	  break;
      }
    }


    if(nor2<Sip_Opt.pf_preco){
      printf("OUTER STOPPING IN ITERATION ********* %d\n", ko+1);
      printf("Norm dir=%f\n", nor2);
      break;
    }

    printf("Dir:\n");
    printarray(dir1, nxsip);
    printf("x0:\n");
    printarray(x0, nxsip);
    printf("g0:\n");
    printarray(g0, nxsip);

    printf("PosDef=%f\n", dual_posdef);
    dual_posdef-=Integral_gxtdgrd(dual_sol, dir1);
    dual_posdef+=Sip_Opt.pf_preco;

    printf("PosDef=%f\n", dual_posdef);

    pterm=dual_pterm(nxsip, x0, ntsip, tsample, FIRST_DERIV, gmf);
    stats.nsip_gpfval++;

    printf("pterm=%f\n", pterm);
    printf("gmf\n");
    printarray(gmf, nxsip);


    if(ko==0){ /* evaluate the lagrangian in x0 */
      memcpy(lg0, g0, nxsip*sizeof(double));
      Integral_gxtgrd(dual_sol, lg0);
      
      printf("First Lag g0:\n");
      printarray(lg0, nxsip);
    }

    if(dual_posdef>=0){
      if(pterm>Sip_Opt.zero){
	/*    	miu=0.0; */
	printf("miu prev=%f\n",miu);
	miu=((dual_posdef/pterm)>miu? 10*(dual_posdef/pterm):10*miu);
	printf("foi ca\n");
      } else {
	/* direction will be the symmetric of merit gradient */
  	for(i=0;i<nxsip;i++)
  	  dir1[i]=-g0[i]-miu*gmf[i];
/*  	miu=1e+8; */
      }
    }
    

    printf("pterm=%.20f\n", pterm);

/*      if(miu>1e+8)miu=1e+8; */

    printf("miu=%f\n", miu);


    for(i=0;i<nxsip;i++)
      gmf[i]=g0[i]+miu*gmf[i];
      
    printf("Gradiente da funcao merito\n");
    printarray(gmf, nxsip);


    dual_armijo(nxsip, x0, dir1, x1, gmf, miu);


    /*      printf("Exit point for Last NPSOL\n"); */
    /*      printarray(dual_sol, (dual_n+ntsipc*(2*dual_n+1))); */


    dummy=eval_sip(SIP_OBJGRD, 0, x1, NULL, g1, NULL, NULL, NULL);
    pterm=dual_pterm(nxsip, x1, ntsip, tsample, NO_DERIV, gmf);

    /* evaluate the lagrangian gradient in new x */
      
    dual_x=x1;
    memcpy(lg1, g1, nxsip*sizeof(double));
    Integral_gxtgrd(dual_sol, lg1);
    dual_x=x0;

    /* lagrangian of x0 with new lag multipliers */
    /*  memcpy(lg0x1, g0, nxsip*sizeof(double)); */
    /*  Integral_gxtgrd(dual_sol, lg0x1); */
    

    if(dual_n!=nxsip)dual_n--;
    if(dual_n<1)dual_n=1;
    dual_sol=NULL;

    printf("x0:\n");
    printarray(x0, nxsip);

    printf("f0=%f\n", eval_sip(SIP_OBJVAL, 0,x0, NULL, NULL, NULL, NULL, NULL));
    printf("Pterm0=%f\n", dual_pterm(nxsip, x0, ntsip, tsample,NO_DERIV, gmf));

    printf("g0:\n");
    printarray(g0, nxsip);

    printf("Lag g0:\n");
    printarray(lg0, nxsip);

    printf("x1:\n");
    printarray(x1, nxsip);

    printf("f1=%f\n", eval_sip(SIP_OBJVAL, 0,x1, NULL, NULL, NULL, NULL, NULL));
    printf("Pterm1=%f\n", dual_pterm(nxsip, x1, ntsip, tsample,NO_DERIV, gmf));

    printf("g1:\n");
    printarray(g1, nxsip);

    printf("Lag g1:\n");
    printarray(lg1, nxsip);

    /* stopping criteria */
    if(dual_stop(nxsip, x0, x1, lg1)){
      printf("OUTER STOPPING IN ITERATION ********* %d\n", ko+1);
      break;
    }
  }

  printf("STOPPING IN ITERATION ********* %d\n", ko+1);  
  memcpy(X, x1, nxsip*sizeof(double));
  printf("Solution found\n");
  printarray(X, nxsip);
  fp=fopen("./results", "a+");
  f0=eval_sip(SIP_OBJVAL, 0, X, NULL, NULL, NULL, NULL, NULL);

  printf("Gradient of objective:\n");
  printarray(g1, nxsip);

  fprintf(fp, " & %d & %d & %d & %d & %.6e \\\\\n",
	  ko+1, stats.nsip_pfval, stats.nsip_contval,
	  stats.nsip_contgrd, f0);
  fclose(fp);


  for(i=0;i<nxsip;i++)
    free(ini_sol[i]);
  free(dual_h);

#if !defined __linux__ && !defined linux
  free(tsample);
  free(dir0);
  free(dir1);
  free(x0);
  free(x1);
  free(g1);
  free(g0);
  free(lg1);
  free(lg0);
  free(gmf);
  free(ini_sol);
#endif

}



/*********************************************************
Evaluate the penalty function
*********************************************************/
double dual_pf(int nx, double *x, int nt, double *t, double miu, double *pt)
{
  double pfval;
  double num, den;
  int i;

  stats.nsip_pfval++;

  pfval=eval_sip(SIP_OBJVAL, 0, x, NULL, NULL, NULL, NULL, NULL);

  printf("dual_pf: objval=%f", pfval);
  
  *pt=0.0;

  if(miu==0.0) return pfval;

  for(i=0; i<ntsipc; i++){
    switch (Sip_Opt.pf_int){
    case PF_GAUSS:
      gaussian(&gxtplus2, TBL, TBU, nx, x, nt, t, i, Sip_Opt.adapt_int_n,
	       &num, &den, NO_DERIV, NULL, miu);
      break;
    case PF_TRAP:
      trapezoid(&gxtplus2, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		i, &num, &den, NO_DERIV, NULL, miu);
      break;
    case PF_ADAPT:
      adapt_trap(&gxtplus2, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		 i, &num, &den, Sip_Opt.adapt_int_error,
		 Sip_Opt.adapt_int_amp, NO_DERIV, NULL, miu);
      break;
    case PF_ADAPT_GAUSS:
      adapt_gauss(&gxtplus2, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		  i, &num, &den, Sip_Opt.adapt_int_error,
		  Sip_Opt.adapt_int_amp, NO_DERIV, NULL, miu);
      break;
    default:
      printf("Unknown integral function\n");
    }

    pfval+=0.5*miu*num;
    *pt+=num;
  }
  printf(" pt=%f\n", *pt);

  return pfval;
}


double dual_pterm(int nx, double *x, int nt, double *t,int dlevel, double *gmf)
{
  double pt, num, den;
  int i, k;

#if defined __linux__ || defined linux
  double gd[nx];
#else
  double *gd;
  gd=mymalloc(nx*sizeof(double));
#endif

  pt=0.0;
  memset(gmf, 0, nxsip*sizeof(double));

  for(i=0; i<ntsipc; i++){
    switch (Sip_Opt.pf_int){
    case PF_GAUSS:
      gaussian(&gxtplus2, TBL, TBU, nx, x, nt, t, i, Sip_Opt.adapt_int_n,
	       &num, &den, dlevel, gd, 0.0);
      break;
    case PF_TRAP:
      trapezoid(&gxtplus2, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		i, &num, &den, dlevel, gd, 0.0);
      break;
    case PF_ADAPT:
      adapt_trap(&gxtplus2, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		 i, &num, &den, Sip_Opt.adapt_int_error,
		 Sip_Opt.adapt_int_amp, dlevel, gd, 0.0);
      break;
    case PF_ADAPT_GAUSS:
      adapt_gauss(&gxtplus2, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		  i, &num, &den, Sip_Opt.adapt_int_error,
		  Sip_Opt.adapt_int_amp, dlevel, gd, 0.0);
      break;
    default:
      printf("Unknown integral function\n");
    }

    pt+=num;
    for(k=0;k<nx;k++)
      gmf[k]+=gd[k];
  }
#if !defined __linux__ && !defined linux
  free(gd);
#endif

  return pt;
}




double Integral_gxt(double *Xn)
{
 int i, j;
 double aux, num, den;

#if defined __linux__ || defined linux
  double tsample[ntsip];
#else
  double *tsample;
  tsample=mymalloc(ntsip*sizeof(double));
#endif
 
 
 aux=0.0;
 
 for(i=0;i<ntsipc;i++){
   for(j=0;j<dual_n;j++){
     aux+=(eval_sip(SIP_CONTVAL, i, dual_x, &Xn[j], NULL, NULL, NULL, NULL)
	   -TCBU[i])*Xn[dual_n+j*ntsipc+i];
   }
   
   adapt_trap(&dual_gxt, TBL, TBU, nxsip, Xn, ntsip, tsample,
	      Sip_Opt.adapt_int_n, i, &num, &den, Sip_Opt.adapt_int_error,
	      Sip_Opt.adapt_int_amp, NO_DERIV, NULL, 0.0);
   aux+=num;
 }

#if !defined __linux__ && !defined linux
  free(tsample);
#endif
 
 return aux;
}


double Integral_unit(double *Xn)
{
 int i, j;
 double aux, num, den;

#if defined __linux__ || defined linux
  double tsample[ntsip];
#else
  double *tsample;
  tsample=mymalloc(ntsip*sizeof(double));
#endif
 
 
 aux=0.0;
 
 for(i=0;i<ntsipc;i++){
   for(j=0;j<dual_n;j++){
     aux+=Xn[dual_n+j*ntsipc+i];
   }
   
   adapt_trap(&unit, TBL, TBU, nxsip, Xn, ntsip, tsample,
	      Sip_Opt.adapt_int_n, i, &num, &den, Sip_Opt.adapt_int_error,
	      Sip_Opt.adapt_int_amp, NO_DERIV, NULL, 0.0);
   aux+=num;
 }

#if !defined __linux__ && !defined linux
  free(tsample);
#endif
 
 return aux;
}




void dual_armijo(int nx, double *x0, double *dir, double *x1, double *grd,
		 double miu)
{
  int l;
  double alpha, dif, phi0, phiomega, nor, gd, pt;

#if defined __linux__ || defined linux
  double tsample[ntsip];
#else
  double *tsample;
  tsample=mymalloc(ntsip*sizeof(double));
#endif


  alpha=1.0;
  phi0=dual_pf(nx, x0, ntsip, tsample, miu, &pt);
  printf("phi0=%f\n", phi0);
  
  gd=0.0;
  nor=0.0;
  for(l=0;l<nx;l++){
    gd+=grd[l]*dir[l];
    nor+=dir[l]*dir[l];
  }
  
  nor=sqrt(nor);

  printf("gd=%f\n", gd);

  
  for(;;){
    for(l=0;l<nx;l++)
      x1[l]=x0[l]+alpha*dir[l];
    phiomega=dual_pf(nx, x1, ntsip, tsample, miu, &pt);
    printf("alpha=%f phiomega=%f\n",alpha, phiomega);
    
    dif=phi0-phiomega;
    if(dif<-(Sip_Opt.armijo)*alpha*gd){
      alpha=alpha/2;
      if(alpha*nor<=1e-8){
	for(l=0;l<nx;l++)
	  x1[l]=x0[l]+alpha*dir[l];
	printf("Step zero %.20f\n", alpha);
#if !defined __linux__ && !defined linux
	free(tsample);
#endif
	return;
      }
    } else {
      printf("Step=%f\n",alpha);
#if !defined __linux__ && !defined linux
      free(tsample);
#endif
      return;
    }
  }
#if !defined __linux__ && !defined linux
  free(tsample);
#endif

}




double Integral_gxtdgrd(double *Xn, double *d)
{
  int i, j, k;
  double dummy, num, den, aux;

#if defined __linux__ || defined linux
  double tsample[ntsip], gd[nxsip];
#else
  double *tsample, *gd;
  tsample=mymalloc(ntsip*sizeof(double));
  gd=mymalloc(nxsip*sizeof(double));
#endif

  
  aux=0.0;
  
  for(i=0;i<ntsipc;i++){ /* for all constraints */

    for(j=0;j<dual_n;j++){ /* for all t variables */

      dummy=eval_sip(SIP_CONTGRD, i, dual_x, &Xn[j], gd, tsample, NULL, NULL);

      for(k=0;k<nxsip;k++){ /* for all x variables */
	aux+=gd[k]* Xn[dual_n+j*ntsipc+i]* d[k];
      }
    }

    adapt_trap(&dual_gxt, TBL, TBU, nxsip, Xn, ntsip, tsample,
  	       Sip_Opt.adapt_int_n, i, &num, &den, Sip_Opt.adapt_int_error,
 	       Sip_Opt.adapt_int_amp, FIRST_DERIV, gd, 0.0);
    
    for(k=0;k<nxsip;k++){ /* for all x variables */
      aux+=gd[k]* d[k];
    }

  }
#if !defined __linux__ && !defined linux
  free(tsample);
  free(gd);
#endif

  return aux;
}






void get_dir_dualQP(double *dir)
{
  double *npsol_bl, *npsol_bu, *npsol_a, *npsol_c, *npsol_cjac, *npsol_clambda;
  double *npsol_objgrd, *npsol_r, *npsol_w;
  double npsol_objf;
  long npsol_n, npsol_nclin, npsol_ncnln, npsol_nrowa, npsol_nrowj;
  long npsol_nrowr;
  long npsol_inform, npsol_iter;
  long *npsol_iw, *npsol_istate, npsol_leniw, npsol_lenw;
  int i, j, k;
  double step;

#if defined __linux__ || defined linux
  double aux[nxsip];
#else
  double *aux;
  aux=mymalloc(nxsip*sizeof(double));
#endif

  
  npsol_n=dual_n+      /* split points */
    ntsipc*dual_n+     /* jumps */
    ntsipc*(dual_n+1); /* slopes */


  if(Sip_Opt.dual_ini_res){
    for(i=0;i<nxsip;i++){ 
      step=(TBU[0]-TBL[0])/(dual_n+1); 
      for(k=0;k<dual_n;k++) 
        dual_sol[k]=TBL[0]+step*(k+1); 
      for(k=dual_n;k<dual_n*(ntsipc+1);k++) 
        dual_sol[k]=1.0; 
      for(k=dual_n*(ntsipc+1);k<dual_n+ntsipc*dual_n+ntsipc*(dual_n+1);k++) 
        dual_sol[k]=1.0; 
    } 
  }



  printf("Dual guess with dual_n=%d npsol_n=%ld\n", dual_n, npsol_n);
/*    printarray(dual_sol, npsol_n); */

  npsol_nclin=dual_n-1; /* linear constraints */
  npsol_nrowa=dual_n-1;
  npsol_a=mymalloc((dual_n-1)*npsol_n*sizeof(double));

  /* build linear constraints */
  if(dual_n>1){
    memset(npsol_a, 0, (dual_n-1)*npsol_n*sizeof(double));
    /*      for(i=1;i<dual_n;i++){ */
    /*        npsol_a[(i-1)*npsol_n+i-1]=-1.0; */
    /*        npsol_a[(i-1)*npsol_n+i]  = 1.0; */ /* t[i]-t[i-1]>=0 */
    /*      } */
    /* tranpose */
    for(i=0;i<dual_n-1;i++){
      npsol_a[i*(dual_n-1)+i]=-1.0;
      npsol_a[(i+1)*(dual_n-1)+i]  = 1.0; /* t[i]-t[i-1]>=0 */
    }
    /*      printf("Linear constraints\n"); */
    /*      printmatrix(npsol_a, npsol_n, dual_n-1); */
    
    /*      printf("Linear constraints\n"); */
    /*      printmatrix(npsol_a, n-1, npsol_n); */
    

  }
    

  npsol_ncnln=0; /* no nonlinear constraints */

  npsol_bl=mymalloc((npsol_n+npsol_nclin)*sizeof(double));
  npsol_bu=mymalloc((npsol_n+npsol_nclin)*sizeof(double));


  /* all omega and zeta variables must be positive;
     except first and last tau */
  for(i=0;i<(npsol_n+npsol_nclin);i++){
    npsol_bl[i]=0.0;
    npsol_bu[i]=+Inf;
  }

  /* tau limits */
  for(i=0;i<dual_n;i++){
    npsol_bl[i]=TBL[0];
    npsol_bu[i]=TBU[0]; /* or +Inf */
  }


  /*    printf("Npsol Lower bounds\n"); */
  /*    printarray(npsol_bl, npsol_n+npsol_nclin); */
  /*    printf("Npsol upper bounds\n"); */
  /*    printarray(npsol_bu, npsol_n+npsol_nclin); */


  /* allocate space for jacobian */
  npsol_nrowj=npsol_ncnln;
  npsol_cjac=mymalloc(npsol_ncnln*sizeof(double));

  /* allocate istate array */
  npsol_istate=mymalloc((npsol_n+npsol_nclin+npsol_ncnln)*sizeof(double));

  /* allocate space for c array */
  npsol_c=mymalloc(npsol_ncnln*sizeof(double));

  /* allocate clambda array */
  npsol_clambda=mymalloc((npsol_n+npsol_nclin+npsol_ncnln)*sizeof(double));

  /* allocate objgrd array */
  npsol_objgrd=mymalloc(npsol_n*sizeof(double));

  /* allocate r matrix */
  npsol_nrowr=npsol_n;
  npsol_r=mymalloc((npsol_n*npsol_nrowr)*sizeof(double));

  /* allocate workspace for npsol */
  npsol_leniw=3*npsol_n+npsol_nclin+2*npsol_ncnln;
  npsol_iw=mymalloc(npsol_leniw*sizeof(double));
  npsol_lenw=2*npsol_n*npsol_n+npsol_n*npsol_nclin+
    2*npsol_n*npsol_ncnln+20*npsol_n+11*npsol_nclin+21*npsol_ncnln;
  npsol_w=mymalloc(npsol_lenw*sizeof(double));

  npoptn_("Derivative level   3", 20L);
  npoptn_("Verify Level      -1", 20L);
  /*    npoptn_("Verify Objective Check At Variable 1", 36L); */
  /*    npoptn_("Major print level 20", 20L); */
  /*    npoptn_("Major iterations limit 999", 26L); */
  /*    npoptn_("Minor print level 20", 20L); */
  /*    npoptn_("Minor iterations limit 999", 26L); */
  npoptn_("Function Precision 1E-4", 23L);
  

  npsol_(&npsol_n, &npsol_nclin, &npsol_ncnln, &npsol_nrowa, &npsol_nrowj,
	 &npsol_nrowr, npsol_a, npsol_bl, npsol_bu, dual_confun, dual_objfun,
	 &npsol_inform, &npsol_iter, npsol_istate, npsol_c, npsol_cjac,
	 npsol_clambda, &npsol_objf, npsol_objgrd, npsol_r, dual_sol, npsol_iw,
	 &npsol_leniw, npsol_w, &npsol_lenw);
  printf("\tNonlinear used %ld iterations Obj=%f Inform=%ld\n",npsol_iter,
  	 npsol_objf, npsol_inform);

  switch(npsol_inform){
  case 2:
  case 3:
  case 7:
  case 9:
    printf("NPSOL returned a sevear error\n Do not know what to do\n");
    break;
  case 4: /* another chance, this might be a warm startup */
    npsol_(&npsol_n, &npsol_nclin, &npsol_ncnln, &npsol_nrowa, &npsol_nrowj,
	   &npsol_nrowr, npsol_a, npsol_bl, npsol_bu, dual_confun, dual_objfun,
	   &npsol_inform, &npsol_iter, npsol_istate, npsol_c, npsol_cjac,
	   npsol_clambda, &npsol_objf, npsol_objgrd, npsol_r, dual_sol,
	   npsol_iw, &npsol_leniw, npsol_w, &npsol_lenw);
    printf("\tNonlinear used %ld iterations Obj=%f Inform=%ld\n",npsol_iter,
	   npsol_objf, npsol_inform);
    break;
  case 6: /* another chance with another initial guess ?*/
    if(!Sip_Opt.dual_ini_res){
      for(i=0;i<nxsip;i++){ 
	step=(TBU[0]-TBL[0])/(dual_n+1); 
	for(k=0;k<dual_n;k++) 
	  dual_sol[k]=TBL[0]+step*(k+1); 
	for(k=dual_n;k<dual_n*(ntsipc+1);k++) 
	  dual_sol[k]=1.0; 
	for(k=dual_n*(ntsipc+1);k<dual_n+ntsipc*dual_n+ntsipc*(dual_n+1);k++) 
	  dual_sol[k]=1.0; 
      } 
      npsol_(&npsol_n, &npsol_nclin, &npsol_ncnln, &npsol_nrowa, &npsol_nrowj,
	     &npsol_nrowr, npsol_a, npsol_bl, npsol_bu, dual_confun,
	     dual_objfun, &npsol_inform, &npsol_iter, npsol_istate, npsol_c,
	     npsol_cjac, npsol_clambda, &npsol_objf, npsol_objgrd, npsol_r,
	     dual_sol, npsol_iw, &npsol_leniw, npsol_w, &npsol_lenw);
      printf("\tNonlinear used %ld iterations Obj=%f Inform=%ld\n",npsol_iter,
	     npsol_objf, npsol_inform);
    }
  }

  
  /*    printf("NPSOL inform=%ld\n", npsol_inform); */
  /*    printf("Exit point for NPSOL\n"); */
  /*    printarray(dual_sol, npsol_n); */
  
  
  /* from dual get primal direction */

  memcpy(aux, dual_xgrd, nxsip*sizeof(double));

  Integral_gxtgrd(dual_sol, aux);

  memset(dir, 0, nxsip*sizeof(double));

  for(i=0;i<nxsip;i++)  /* matrix h is simmetric */
    for(j=0;j<nxsip;j++)
      dir[i]+=-aux[j]*dual_h[i*nxsip+j];

  /*    printf("dir\n"); */
  /*    printarray(dir, nxsip); */

  dual_posdef=0.0;
  for(i=0;i<nxsip;i++)
    dual_posdef+=dir[i]*aux[i];

  
  /* free space */
  free(npsol_bl);
  free(npsol_bu);
  free(npsol_cjac);
  free(npsol_istate);
  free(npsol_a);
  free(npsol_c);
  free(npsol_clambda);
  free(npsol_objgrd);
  free(npsol_r);
  free(npsol_iw);
  free(npsol_w);

#if !defined __linux__ && !defined linux
  free(aux);
#endif
  
}



void dual_objfun(long *MODE, long *N, double *Xn, double *F, double *G,
		 long *NSTATE)
{
  double dummy, num, den, c, factor;
  int i, j, k, int_n;

#if defined __linux__ || defined linux
  double aux[nxsip], haux[nxsip], dxhpq[nxsip*dual_n*ntsipc];
  double dthpq[ntsip*dual_n*ntsipc];
  double hpq[dual_n*ntsipc], tp[nxsip*dual_n];
  double tsample[ntsip], RX[nxsip*nxsip], RT[ntsip*ntsip], RXT[nxsip*ntsip];
  double gd[nxsip];
#else
  double *aux, *haux, *dxhpq, *dthpq, *hpq;
  double *tp, *tsample, *RX, *RT, *RXT, *gd;
  aux=mymalloc(nxsip*sizeof(double));
  gd=mymalloc(nxsip*sizeof(double));
  haux=mymalloc(nxsip*sizeof(double));
  dxhpq=mymalloc(nxsip*dual_n*ntsipc*sizeof(double));
  dthpq=mymalloc(ntsip*dual_n*ntsipc*sizeof(double));
  hpq=mymalloc(dual_n*ntsipc*sizeof(double));
  tp=mymalloc(nxsip*dual_n*sizeof(double));
  tsample=mymalloc(ntsip*sizeof(double));
  RX=mymalloc(nxsip*nxsip*sizeof(double));
  RT=mymalloc(ntsip*ntsip*sizeof(double));
  RXT=mymalloc(nxsip*ntsip*sizeof(double));
#endif

/*    printf("NPSOL objfun called with X=\n"); */
/*    printarray(Xn, dual_n+ntsipc*dual_n+ntsipc*(dual_n+1)); */

/*    if (*MODE != 2){ */
/*    printf("NPSOL objfun called with MODE=%ld\n",*MODE); */
/*    } */

/*    printf("OBJFUN CALLED\n"); */
/*    printf("dual_n=%d\ndual_x\n",dual_n); */
/*    printarray(dual_x, nxsip); */
/*    printf("npsol_x\n"); */
/*    printarray(Xn, dual_n+ntsipc*dual_n+ntsipc*(dual_n+1)); */

  memcpy(aux, dual_xgrd, nxsip*sizeof(double));
  
  /* add the integral */

  c=0.0;

  for(i=0;i<ntsipc;i++){ /* for all constraints */

    for(j=0;j<dual_n;j++){ /* for all t variables */

      dummy=eval_sip(SIP_CONTGRD, i, dual_x, &Xn[j],
		     &dxhpq[nxsip*j+nxsip*dual_n*i],
		     &dthpq[ntsip*j+ntsip*dual_n*i], NULL, NULL);

      for(k=0;k<nxsip;k++){ /* for all x variables */
	aux[k]+=dxhpq[nxsip*j+nxsip*dual_n*i+k]* Xn[dual_n+j*ntsipc+i];
      }
    }



    adapt_trap(&dual_gxt, TBL, TBU, nxsip, Xn, ntsip, tsample,
  	       Sip_Opt.adapt_int_n, i, &num, &den, Sip_Opt.adapt_int_error,
 	       Sip_Opt.adapt_int_amp, FIRST_DERIV, gd, 0.0);
    
    c+=num;
    for(k=0;k<nxsip;k++){ /* for all x variables */
      aux[k]+=gd[k];
    }

  }

  memset(haux, 0, nxsip*sizeof(double));
  
  for(i=0;i<nxsip;i++)  /* matrix h is simmetric */
    for(j=0;j<nxsip;j++)
      haux[i]+=aux[j]*dual_h[j*nxsip+i];
  
  /* haux used for derivatives and objective function computation */

  memset(tp, 0, nxsip*dual_n*sizeof(double));
  
  for(i=0;i<ntsipc;i++){ /* for all constraints */

    for(j=0;j<dual_n;j++){ /* for all t variables */

      hpq[j+i*dual_n]=eval_sip(SIP_CONTVAL, i, dual_x, &Xn[j], NULL,
			       NULL, NULL, NULL)-TCBU[i];
      c+=hpq[j+i*dual_n]*Xn[dual_n+j*ntsipc+i];

      if(*MODE!=0){
	eval_sip(SIP_CONTHES, i, NULL, NULL, RX, RT, RXT, NULL);
	/* only one t variable */
	for(k=0;k<nxsip;k++)
	  tp[j*nxsip+k]+=RXT[k]*Xn[dual_n+j*ntsipc+i];
      }

    }
  }

  if(*MODE!=1){
    *F=0.0;
    for(i=0;i<nxsip;i++)
      *F+=0.5*haux[i]*aux[i];
    
    *F-=c;
  }
  
  if(*MODE!=0){

    /* derivatives w.r.t. the t variables */

    for(j=0;j<dual_n;j++){
      
      memset(gd, 0, nxsip*sizeof(double));
      dummy=0.0;
      
      for(i=0;i<ntsipc;i++){
	factor=(Xn[dual_n+dual_n*ntsipc+j*ntsipc+i]
		-Xn[dual_n+dual_n*ntsipc+(j+1)*ntsipc+i]);
	for(k=0;k<nxsip;k++)
	  gd[k]+=factor*dxhpq[nxsip*j+nxsip*dual_n*i+k];
	dummy+=factor*hpq[dual_n*i+j];
      }
      
      G[j]=0.0;
      for(k=0;k<nxsip;k++)
	G[j]+=haux[k]*(tp[nxsip*j+k]+gd[k]);
      
      for(i=0;i<ntsipc;i++)
	/* just one t variable */
	G[j]-=dthpq[ntsip*dual_n*i+j*ntsip]*Xn[dual_n+j*ntsipc+i];
      G[j]-=dummy;

    }
    
    /* derivatives w.r.t. the h variables */

    for(i=0;i<ntsipc;i++)

      for(j=0;j<dual_n;j++){

	G[dual_n+j*ntsipc+i]=0.0;
	for(k=0;k<nxsip;k++)
	  G[dual_n+j*ntsipc+i]+=haux[k]*dxhpq[nxsip*j+nxsip*dual_n*i+k];
	G[dual_n+j*ntsipc+i]-=hpq[j+i*dual_n]; 
      }

    /* derivatives w.r.t. the w variables */

    for(i=0;i<ntsipc;i++){
      
      /* for w[0] and w[dual_n] */
      
      if((Xn[0]-TBL[0])>0){
	if(((Xn[0]-TBL[0])/Sip_Opt.adapt_int_amp)>Sip_Opt.adapt_int_n){
	  int_n=Sip_Opt.adapt_int_n;
	} else {
	  int_n=(Xn[0]-TBL[0])/Sip_Opt.adapt_int_amp;
	}
	if(int_n<2)int_n=2;
	adapt_trap(&gxt, TBL, &Xn[0], nxsip, dual_x, ntsip, tsample,
		   int_n, i, &num, &den, Sip_Opt.adapt_int_error,
		   Sip_Opt.adapt_int_amp, FIRST_DERIV, gd, 0.0);
	
	G[dual_n+ntsipc*dual_n+i]=0.0;
	for(k=0;k<nxsip;k++)
	  G[dual_n+ntsipc*dual_n+i]+=haux[k]*gd[k];
	G[dual_n+ntsipc*dual_n+i]-=num;
      } else {
	G[dual_n+ntsipc*dual_n+i]=0.0;
      }
    
      if((TBU[0]-Xn[dual_n-1])>0){
	if(((TBU[0]-Xn[dual_n-1])/Sip_Opt.adapt_int_amp)>
	   Sip_Opt.adapt_int_n){
	  int_n=Sip_Opt.adapt_int_n;
	} else {
	  int_n=(TBU[0]-Xn[dual_n-1])/Sip_Opt.adapt_int_amp;
	}
	if(int_n<2)int_n=2;
	adapt_trap(&gxt, &Xn[dual_n-1], TBU, nxsip, dual_x, ntsip, tsample,
		   int_n, i, &num, &den, Sip_Opt.adapt_int_error,
		   Sip_Opt.adapt_int_amp, FIRST_DERIV, gd, 0.0);

	G[dual_n+2*ntsipc*dual_n+i]=0.0;
	for(k=0;k<nxsip;k++)
	  G[dual_n+2*ntsipc*dual_n+i]+=haux[k]*gd[k];
	G[dual_n+2*ntsipc*dual_n+i]-=num;
      } else {
	G[dual_n+2*ntsipc*dual_n+i]=0.0;
      }
      
      /* for w[j] j!=0,dual_n */
      
      for(j=1;j<dual_n;j++){
	
	if((Xn[j]-Xn[j-1])>0){
	  if(((Xn[j]-Xn[j-1])/Sip_Opt.adapt_int_amp)>Sip_Opt.adapt_int_n){
	    int_n=Sip_Opt.adapt_int_n;
	  } else {
	    int_n=(Xn[j]-Xn[j-1])/Sip_Opt.adapt_int_amp;
	  }
	  if(int_n<2)int_n=2;
	  adapt_trap(&gxt, &Xn[j-1],&Xn[j], nxsip, dual_x, ntsip, tsample,
		     int_n, i, &num, &den, Sip_Opt.adapt_int_error,
		     Sip_Opt.adapt_int_amp, FIRST_DERIV, gd, 0.0);

	  G[dual_n+ntsipc*dual_n+j*ntsipc+i]=0.0;
	  for(k=0;k<nxsip;k++)
	    G[dual_n+ntsipc*dual_n+j*ntsipc+i]+=haux[k]*gd[k];
	  G[dual_n+ntsipc*dual_n+j*ntsipc+i]-=num;
	} else {
	  G[dual_n+ntsipc*dual_n+j*ntsipc+i]=0.0;
	}
      }
      
    }
  }
#if !defined __linux__ && !defined linux
  free(aux);
  free(haux);
  free(dxhpq);
  free(dthpq);
  free(hpq);
  free(tp);
  free(tsample);
  free(RX);
  free(RT);
  free(RXT);
  free(gd);
#endif
  
}


void dual_confun(long *MODE, long *NCNLN, long *N, long *NROWJ, long *NEEDC,
		 double *X0, double *C, double *CJAC, long *NSTATE)
{
  /* problem without nonlinear constraints */

  printf("Major error.... This function should not be called.\n");
}




void Integral_gxtgrd(double *Xn, double *aux)
{
  int i, j, k;
  double dummy, num, den;

#if defined __linux__ || defined linux
  double tsample[ntsip], gd[nxsip];
#else
  double *tsample, *gd;
  gd=mymalloc(nxsip*sizeof(double));
  tsample=mymalloc(ntsip*sizeof(double));
#endif


  /* aux vector was set, just add the integral */

  for(i=0;i<ntsipc;i++){ /* for all constraints */

    for(j=0;j<dual_n;j++){ /* for all t variables */

      dummy=eval_sip(SIP_CONTGRD, i, dual_x, &Xn[j], gd, tsample, NULL, NULL);

      for(k=0;k<nxsip;k++){ /* for all x variables */
	aux[k]+=gd[k]* Xn[dual_n+j*ntsipc+i];
      }
    }



    adapt_trap(&dual_gxt, TBL, TBU, nxsip, Xn, ntsip, tsample,
  	       Sip_Opt.adapt_int_n, i, &num, &den, Sip_Opt.adapt_int_error,
 	       Sip_Opt.adapt_int_amp, FIRST_DERIV, gd, 0.0);
    
    for(k=0;k<nxsip;k++){ /* for all x variables */
      aux[k]+=gd[k];
    }

  }
#if !defined __linux__ && !defined linux
  free(tsample);
  free(gd);
#endif

}



/******************************************************
Function for integral
******************************************************/
double dual_gxt(double *x, double *t, int j, int dlevel, double *grad,
		double miu)
{
  double dummy, tval;
  int i, k;

#if defined __linux__ || defined linux
  double gt[ntsip];
#else
  double *gt;
  gt=mymalloc(ntsip*sizeof(double));
#endif


  /* x contains the NPSOL variables, use dual_x for x value */

  if(dlevel==FIRST_DERIV)
    dummy=eval_sip(SIP_CONTGRD, j, dual_x, t, grad, gt, NULL, NULL);

  tval=eval_sip(SIP_CONTVAL, j, dual_x, t, NULL, NULL, NULL, NULL)-TCBU[j];

  for(i=0;i<dual_n;i++){
    if(t[0]<x[i]){
      if(dlevel==FIRST_DERIV)
	for(k=0;k<nxsip;k++)
	  grad[k]*=x[dual_n+dual_n*ntsipc+i*ntsipc+j];
      tval*=x[dual_n+dual_n*ntsipc+i*ntsipc+j];
      return tval;
    }
  }
  
  /* i=dual_n */
  if(dlevel==FIRST_DERIV)
    for(k=0;k<nxsip;k++)
      grad[k]*=x[dual_n+dual_n*ntsipc+i*ntsipc+j];
  tval*=x[dual_n+dual_n*ntsipc+i*ntsipc+j];

#if !defined __linux__ && !defined linux
  free(gt);
#endif

  return tval;
}


double unit(double *x, double *t, int j, int dlevel, double *grad,
	    double miu)
{
  int i;

  /* x contains the NPSOL variables, use dual_x for x value */

  for(i=0;i<dual_n;i++)
    if(t[0]<x[i])
      return x[dual_n+dual_n*ntsipc+i*ntsipc+j];
  
  /* i=dual_n */
  return x[dual_n+dual_n*ntsipc+i*ntsipc+j];
}



/*******************************
check stopping criteria
*******************************/
int dual_stop(int nx, double *x0, double *x1, double *g1)
{
  int i;
  double aux1, aux2, aux3;

  aux1=aux2=0;
  for(i=0;i<nx;i++){
    aux1+=pow((x1[i]-x0[i]),2.0);
    aux2+=pow(x1[i],2.0);
  }

  if(sqrt(aux2)<=Sip_Opt.zero){
    aux3=sqrt(aux1);
  } else {
    aux3=sqrt(aux1)/sqrt(aux2);
  }

  if(aux3>Sip_Opt.pf_preco){
        printf("Not stopping in outer iteration Diff = %f\n", aux3);
	return 0;
  }


  if(!memcmp(x0,x1,nx*sizeof(double))){
    printf("\n\n*******************************************************\n");
    printf("* NO PROGRESS IN SOLUTION. SOLUTION MAY BE UNFEASIBLE *\n");
    printf("*******************************************************\n\n\n");
    return 1;
  }

  /*
  aux1=0;
  for(i=0;i<nx;i++)
    aux1+=pow(g1[i],2.0);
  if(aux1>Sip_Opt.pf_preco*100){
    printf("Not stopping in outer iteration Norm Lag = %f\n", aux1);
    return 0;
  }
  */

  return 1;
}


#else

void dual_lags()
{

  printf("\n\nNPSOL not available\nPlease read the user manual about it\n");
  printf("Aborting....\n");
  exit(1);

}

#endif
