#include "lpfs.h"
#include "int.h"

extern STATS stats;


void penaltylfs()
{
  double f0, f1, norg;
  int ko, ki, i, ieps, j, l, tki, singular;
  double miu,  pt;  /* penalty parameter */
  FILE *fp;


#if defined __linux__ || defined linux
  double tsample[ntsip];
  double dir[nxsip];
  double h[nxsip*nxsip];
  double x0[nxsip], x1[nxsip], g1[nxsip], g0[nxsip], y[nxsip];
  double yeps[nxsip];
  double lag[ntsipc], cs[ntsipc], jac[ntsipc*nxsip];
  double A[ntsipc*ntsipc], B[ntsipc], newlag[ntsipc];
#else
  double *tsample, *dir, *h, *x0, *x1, *g1, *g0, *y, *yeps;
  double *lag, *cs, *jac, *A, *B, *newlag;
  tsample=mymalloc(ntsip*sizeof(double));
  dir=mymalloc(nxsip*sizeof(double));
  h=mymalloc(nxsip*nxsip*sizeof(double));
  x0=mymalloc(nxsip*sizeof(double));
  x1=mymalloc(nxsip*sizeof(double));
  g1=mymalloc(nxsip*sizeof(double));
  g0=mymalloc(nxsip*sizeof(double));
  y=mymalloc(nxsip*sizeof(double));
  yeps=mymalloc(nxsip*sizeof(double));
  lag=mymalloc(ntsipc*sizeof(double));
  cs=mymalloc(ntsipc*sizeof(double));
  jac=mymalloc(nxsip*ntsipc*sizeof(double));
  A=mymalloc(ntsipc*ntsipc*sizeof(double));
  B=mymalloc(ntsipc*sizeof(double));
  newlag=mymalloc(ntsipc*sizeof(double));
#endif



  miu=Sip_Opt.startmiu;

  tki=0;
  ieps=0;

  memcpy(x1, X, nxsip*sizeof(double)); /* x1 with initial guess */
  memcpy(yeps, X, nxsip*sizeof(double));

  for(i=0;i<ntsipc;i++)
    lag[i]=1.0;

  for(ko=0;ko<Sip_Opt.maxitero;ko++){


    /* y with initial guess for inner iter */
    memcpy(y, x1, nxsip*sizeof(double));

    printf("STARTING INNER ITERATIONS ********** miu=%f, epsilon=%f\n",
	   miu, Sip_Opt.pf_epsilon);

    /* Get new penalty value, miu has changed */
    gpenaltylf(nxsip, x1, ntsip, tsample, ntsipc, lag, miu, g1, &f1, &pt, cs,
	       jac);

    
    for(ki=0;ki<Sip_Opt.maxiteri;ki++){ /* Quasi-Newton inner iteration */

      /* Set Hessian inverse */
      set_h(h, ki, nxsip, x1, x0, g1, g0);

      /*      printf("Hessian inverse\n");
	      printmatrix(h, nxsip, nxsip);*/
    
    
      memcpy(x0, x1, nxsip*sizeof(double));
      memcpy(g0, g1, nxsip*sizeof(double));
      f0=f1;

      get_dir(nxsip, x0, h, g0, dir);

      printf("X: ");
      printarray(x0, nxsip);
      printf("Penalty %f\n", f0);
      printf("Gradient: ");
      printarray(g0, nxsip);
      printf("Direction: ");
      printarray(dir, nxsip);

      if(Sip_Opt.ScaleDir)
	scale_dir(nxsip, dir);

      switch (Sip_Opt.linesearch){
      default:
	printf("Unknown LineSearch\nUsing Armijo\n");
      case ARMIJO:
	larmijo(nxsip, x0, dir, x1, g0, miu, ntsipc, lag);
	break;
      case ARGO:
	largo(nxsip, x0, dir, x1, g0, miu, ntsipc, lag);
	break;
      case WOLFE:
	lwolfe(nxsip, x0, dir, x1, g0, miu, ntsipc, lag);
	break;
      };

      printf("X: ");
      printarray(x1, nxsip);

      gpenaltylf(nxsip, x1, ntsip, tsample, ntsipc, lag, miu, g1, &f1, &pt, cs,
		 jac);

      printf("Penalty terms %f %f constraints %f\n", f0, f1, pt);


      /* inner stopping criteria */
      if(pf_i_stop(nxsip, x0, x1, g1, pt)){
	printf("INNER STOPPING IN ITERATION ********* %d\n", ki+1);
	break;
      }
    }

    tki+=ki+1;

    for(i=0;i<ntsipc;i++)
      printf("Cs[%d]=%f  Lag[%d]=%f\n", i, cs[i], i, lag[i]);


    /* update multiplier estimates */
    switch(Sip_Opt.multipliersUPD){
    default:
      printf("Unknown multipliers update method\n");
      printf("Using standard multiplier method\n");
    case MULTI:
      if(Sip_Opt.pf_type!=PF_GXTEXPTEO){
	for(i=0;i<ntsipc;i++)
	  lag[i]+=miu*cs[i];
      } else {
	for(i=0;i<ntsipc;i++)
	  lag[i]*=cs[i];
      }
      break;
    case GENINV:
      printf("Jacobian:\n");
      printmatrix(jac, ntsipc, nxsip);
      /* A= jac*jacT , simmetric matrix */
      memset(A, 0, sizeof(double)*ntsipc*ntsipc);
      for(i=0;i<ntsipc;i++){                          /* optimize */
	for(j=0;j<ntsipc;j++)
	  for(l=0;l<nxsip;l++)
	    A[i*ntsipc+j]+=jac[i*nxsip+l]*jac[j*nxsip+l];
      }

      /*        printf("Prod:\n"); */
      /*        printmatrix(A, ntsipc, ntsipc); */


      /* B=jac*g1 */
      memset(B, 0, sizeof(double)*ntsipc);
      for(i=0;i<ntsipc;i++)
	for(l=0;l<nxsip;l++)
	  B[i]+=jac[i*nxsip+l]*g1[l];
      
      printf("g1:\n");
      printarray(g1, nxsip);

      printf("B:\n");
      printarray(B, ntsipc);

      egpp(ntsipc, A, B, newlag, &singular);

      /*        printf("U:\n"); */
      /*        printmatrix(A, ntsipc, ntsipc); */
      /*        printf("Bu:\n"); */
      /*        printarray(B, ntsipc); */
      /*        printf("Sol:\n"); */
      /*        printarray(newlag, ntsipc); */

      if(!singular){
	memcpy(lag, newlag, sizeof(double)*ntsipc);
	printf("New Lagrange multipliers:\n");
	printarray(lag, ntsipc);
      }
      break;
    }


    for(i=0;i<ntsipc;i++)
      printf("Lag[%d]=%f\n", i, lag[i]);



/*      if((pt-ntsipc*Sip_Opt.pf_tau)>Sip_Opt.zero){ */
    if(pt>Sip_Opt.zero){
      printf("Increasing penalty\n");
      miu*=Sip_Opt.miufactor;
      if(miu>1e+8)
	break;
    } else {
      /*        if(Sip_Opt.pf_type==PF_GXTPLUS2) */
      /*  	break; */
      if(ieps>0){
	if(pf_eps_stop(nxsip, yeps, x1)){
	  break;
	}
      }
      ieps++;
      memcpy(yeps, x1,    nxsip*sizeof(double));
      printf("Decreasing epsilon\n");
      Sip_Opt.pf_epsilon/=10;
      Sip_Opt.pf_tau/=10;
      if(Sip_Opt.pf_epsilon<1e-8)
	break;
    }


    /* outer stopping criteria */
    /*      if(pf_o_stop(nxsip, y, x1, g1, pt)){ */
    /*        printf("OUTER STOPPING IN ITERATION ********* %d\n", ko+1); */
    /*        break; */
    /*      } */
  }

  printf("STOPPING IN ITERATION ********* %d\n", ko+1);  
  memcpy(X, x1, nxsip*sizeof(double));
  printf("Solution found\n");
  printarray(X, nxsip);
  printf("Penalty function value\n");
/*printf("%f\n",penaltylf(nxsip, x1, ntsip, tsample, ntsipc, lag, miu,&pt,cs,*/
/*  			  NO_DERIV, NULL, NULL)); */
  printf("Lagrange multipliers:\n");
  printarray(lag, ntsipc);

  fp=fopen("./results", "a+");
  f0=eval_sip(SIP_OBJVAL, 0, X, NULL, NULL, NULL, NULL, NULL);

  printf("Gradient of penalty:\n");
  printarray(g1, nxsip);

  norg=0.0;
  for(i=0;i<nxsip;i++)
    norg+=g1[i]*g1[i];
  norg=sqrt(norg);

  eval_sip(SIP_OBJGRD, 0, x1, NULL, g1, NULL, NULL, NULL);

  printf("Gradient of objective:\n");
  printarray(g1, nxsip);
  

/*  fprintf(fp, " & %d & %d & %d & %d & %d & %d & %.4e & %.4e & %.2e\\\\\n", */
/*   tki, ko+1, stats.nsip_objval, stats.nsip_objgrd, stats.nsip_contval, */
/*   stats.nsip_contgrd, f0, f1, norg); */

  fprintf(fp, " & %d & %d & %d & %d & %.6e & %.1e & %.1e \\\\\n",
	  tki, ko+1, stats.nsip_pfval, stats.nsip_gpfval, f0, miu,  Sip_Opt.pf_epsilon);
  fclose(fp);

#if !defined __linux__ && !defined linux
  free(tsample);
  free(dir);
  free(h);
  free(x0);
  free(x1);
  free(g1);
  free(g0);
  free(y);
  free(yeps);
  free(lag);
  free(cs);
  free(jac);
  free(A);
  free(B);
  free(newlag);
#endif

}


/*********************************************************
Evaluate the penalty function
*********************************************************/
double penaltylf(int nx, double *x, int nt, double *t, int nl, double *lag,
		 double miu, double *pt, double *cs, int dlevel, double *grad,
		 double *jac)
{
  double pfval, num, lagnum, den, dummy;
  int i, k;

#if defined __linux__ || defined linux
  double gd[nx], laggd[nx];
#else
  double *gd, *laggd;
  gd=mymalloc(nx*sizeof(double));
  laggd=mymalloc(nx*sizeof(double));
#endif

  stats.nsip_pfval++;


  pfval=0.0;
  *pt=0.0;
  if(dlevel==FIRST_DERIV){
    memset(grad, 0, sizeof(double)*nx);
    if(Sip_Opt.multipliersUPD==GENINV)
      memset(jac, 0, sizeof(double)*nx*ntsipc);
  }



  for(i=0; i<ntsipc; i++){

    if(Sip_Opt.pf_type!=PF_GXTTEO) /* for Lagrangian */
      adapt_trap(&gxtteo, TBL, TBU, nx, x, nt, t,
		 Sip_Opt.adapt_int_n, i, &lagnum, &den,
		 Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		 dlevel, laggd, miu);
    
    switch (Sip_Opt.pf_type){
    case PF_GXTPLUS:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxtplus, TBL, TBU, nx, x, nt, t, i, Sip_Opt.adapt_int_n,
		 &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxtplus, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxtplus, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		   i, &num, &den, Sip_Opt.adapt_int_error,
		   Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxtplus, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		    i, &num, &den, Sip_Opt.adapt_int_error,
		    Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_GXTPLUS2:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxtplus2, TBL, TBU, nx, x, nt, t, i, Sip_Opt.adapt_int_n,
		 &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxtplus2, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxtplus2, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		   i, &num, &den, Sip_Opt.adapt_int_error,
		   Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxtplus2, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		    i, &num, &den, Sip_Opt.adapt_int_error,
		    Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_GXTEPSILON:
      Sip_Opt.pf_epsilon=miu;
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxtepsilon, TBL, TBU, nx, x, nt, t, i, Sip_Opt.adapt_int_n,
		 &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxtepsilon, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxtepsilon, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		   i, &num, &den, Sip_Opt.adapt_int_error,
		   Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxtepsilon, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		    i, &num, &den, Sip_Opt.adapt_int_error,
		    Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_GXTEXPPLUS:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxtexpplus, TBL, TBU, nx, x, nt, t, i, Sip_Opt.adapt_int_n,
		 &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxtexpplus, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxtexpplus, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		   i, &num, &den, Sip_Opt.adapt_int_error,
		   Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxtexpplus, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		    i, &num, &den, Sip_Opt.adapt_int_error,
		    Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_GXTEXPEPSILON:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxtexpepsilon, TBL, TBU, nx, x, nt, t, i,
		 Sip_Opt.adapt_int_n, &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxtexpepsilon, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxtexpepsilon, TBL, TBU, nx, x, nt, t,
		   Sip_Opt.adapt_int_n, i, &num, &den,
		   Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		   dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxtexpepsilon, TBL, TBU, nx, x, nt, t,
		    Sip_Opt.adapt_int_n, i, &num, &den,
		    Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		    dlevel, gd, miu);
	break;

      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_GXTTEO:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxtteo, TBL, TBU, nx, x, nt, t, i,
		 Sip_Opt.adapt_int_n, &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxtteo, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxtteo, TBL, TBU, nx, x, nt, t,
		   Sip_Opt.adapt_int_n, i, &num, &den,
		   Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		   dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxtteo, TBL, TBU, nx, x, nt, t,
		    Sip_Opt.adapt_int_n, i, &num, &den,
		    Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp,
		    dlevel, gd, miu);
	break;

      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_GXTEXPTEO:
/*        switch (Sip_Opt.pf_int){ */
/*        case PF_GAUSS: */
/*  	gaussian(&gxtexpteo, TBL, TBU, nx, x, nt, t, i, */
/*  		 Sip_Opt.adapt_int_n, &num, &den, dlevel, gd, miu); */
/*  	break; */
/*        case PF_TRAP: */
/*  	trapezoid(&gxtexpteo, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int, */
/*  		  i, &num, &den, dlevel, gd, miu); */
/*  	break; */
/*        case PF_ADAPT: */
/*  	adapt_trap(&gxtexpteo, TBL, TBU, nx, x, nt, t, */
/*  		   Sip_Opt.adapt_int_n, i, &num, &den, */
/*  		   Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp, */
/*  		   dlevel, gd, miu); */
/*  	break; */
/*        case PF_ADAPT_GAUSS: */
/*  	adapt_gauss(&gxtexpteo, TBL, TBU, nx, x, nt, t, */
/*  		    Sip_Opt.adapt_int_n, i, &num, &den, */
/*  		    Sip_Opt.adapt_int_error, Sip_Opt.adapt_int_amp, */
/*  		    dlevel, gd, miu); */
/*  	break; */

/*        default: */
/*  	printf("Unknown integral function\n"); */
/*        } */

      /* I will compute the penalty later */


      break;
    case PF_GXTHART:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxthart, TBL, TBU, nx, x, nt, t, i, Sip_Opt.adapt_int_n,
		 &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxthart, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxthart, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		   i, &num, &den, Sip_Opt.adapt_int_error,
		   Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxthart, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		    i, &num, &den, Sip_Opt.adapt_int_error,
		    Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      default:
	printf("Unknown integral function\n");
      }
      break;
    case PF_GXTGROSS:
      switch (Sip_Opt.pf_int){
      case PF_GAUSS:
	gaussian(&gxtgross, TBL, TBU, nx, x, nt, t, i, Sip_Opt.adapt_int_n,
		 &num, &den, dlevel, gd, miu);
	break;
      case PF_TRAP:
	trapezoid(&gxtgross, TBL, TBU, nx, x, nt, t, Sip_Opt.n_int,
		  i, &num, &den, dlevel, gd, miu);
	break;
      case PF_ADAPT:
	adapt_trap(&gxtgross, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		   i, &num, &den, Sip_Opt.adapt_int_error,
		   Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      case PF_ADAPT_GAUSS:
	adapt_gauss(&gxtgross, TBL, TBU, nx, x, nt, t, Sip_Opt.adapt_int_n,
		    i, &num, &den, Sip_Opt.adapt_int_error,
		    Sip_Opt.adapt_int_amp, dlevel, gd, miu);
	break;
      default:
	printf("Unknown integral function\n");
      }
      break;
    default:
      printf("Unknown Penalty integral function\n");
    }

    if(Sip_Opt.pf_type!=PF_GXTEXPTEO){
      *pt+=num-Sip_Opt.pf_tau;
    } else {
      *pt+=lagnum-Sip_Opt.pf_tau;
    }

    if(dlevel==FIRST_DERIV && Sip_Opt.multipliersUPD==GENINV)
      memcpy(&(jac[i*nx]), gd, sizeof(double)*nx);

    switch (Sip_Opt.pf_type){
    case PF_GXTTEO:
      cs[i]=num;
      pfval+=lag[i]*(cs[i]-Sip_Opt.pf_tau)+0.5*miu 
  	*pow(cs[i]-Sip_Opt.pf_tau,2.0);
      if(dlevel==FIRST_DERIV){
	for(k=0;k<nx;k++)
  	  grad[k]+=lag[i]*gd[k]+miu*gd[k]*(cs[i]-Sip_Opt.pf_tau);
      }
      break;
    case PF_GXTPLUS2:
      cs[i]=0;
      for(k=0;k<nx;k++){
	if(fabs(laggd[k])>Sip_Opt.zero)
	  cs[i]+=gd[k]/(2*nxsip*laggd[k]);
/*  	printf("laggd[%d]=%.30f\n  gd[%d]=%f\n",  */
/*    	       k, laggd[k], k, gd[k]); */
      }
/*        printf("cs[%d]=%f\n", i, cs[i]); */

      pfval+=lag[i]*(lagnum-Sip_Opt.pf_tau)+0.5*miu*num;
      if(dlevel==FIRST_DERIV){
	for(k=0;k<nx;k++)
	  grad[k]+=lag[i]*laggd[k]+0.5*miu*gd[k];
      }
      break;
    case PF_GXTEXPTEO:
      cs[i]=exp(miu*(lagnum-Sip_Opt.pf_tau));
      pfval+=lag[i]*(cs[i]-1)/miu;
      if(dlevel==FIRST_DERIV){
	for(k=0;k<nx;k++)
	  grad[k]+=lag[i]*laggd[k]*cs[i];
      }
      break;

/*        cs[i]=num*(num+TBU[0]-TBL[0]); */
/*        pfval+=lag[i]*(lagnum-Sip_Opt.pf_tau)+0.5*miu*num*num; */
/*        if(dlevel==FIRST_DERIV){ */
/*  	for(k=0;k<nx;k++) */
/*  	  grad[k]+=lag[i]*laggd[k]+miu*num*gd[k]; */
/*        } */
/*        break; */
    case PF_GXTPLUS:
      pfval+=-lag[i]*(cs[i])+0.5*miu*pow(cs[i], 2.0);
      if(dlevel==FIRST_DERIV){
	for(k=0;k<nx;k++)
	  grad[k]+=-lag[i]*gd[k]+miu*gd[k]*cs[i];
      }
      break;
    case PF_GXTHART:
    case PF_GXTGROSS:
      pfval+=-lag[i]*(cs[i])+0.5*cs[i]/miu;
      if(dlevel==FIRST_DERIV){
	for(k=0;k<nx;k++)
	  grad[k]+=-lag[i]*gd[k]+0.5*gd[k]/miu;
      }
      break;
    default:
      pfval+=-lag[i]*(cs[i])+miu*cs[i];
      if(dlevel==FIRST_DERIV){
	for(k=0;k<nx;k++)
	  grad[k]+=-lag[i]*gd[k]+miu*gd[k];
      }
      break;
    }
  }   

  pfval+=eval_sip(SIP_OBJVAL, 0, x, NULL, NULL, NULL, NULL, NULL);
  
  if(dlevel==FIRST_DERIV){
    dummy=eval_sip(SIP_OBJGRD, 0, x, NULL, gd, NULL, NULL, NULL);
    for(k=0;k<nx;k++)
      grad[k]+=gd[k];
  }

#if !defined __linux__ && !defined linux
  free(gd);
  free(laggd);
#endif


  return pfval;
}


/****************************************************
gradient of penalty with respect to x
Arguments are similar of the penalty function
dx   - gradient
****************************************************/
void gpenaltylf(int nx, double *x, int nt, double *t, int nl, double *lag,
		double miu, double *dx, double *fx, double *pt, double *cs,
		double *jac)
{
  double fx1; /*, fx2;*/
  int i, l;

#if defined __linux__ || defined linux
  double x1[nx], cs1[nl];
#else
  double *x1, *cs1;
  x1=mymalloc(nx*sizeof(double));
  cs1=mymalloc(nl*sizeof(double));
#endif


  stats.nsip_gpfval++;

  *fx=penaltylf(nx, x, nt, t, nl, lag, miu, pt, cs, Sip_Opt.derivatives, dx,
		jac);
  if(Sip_Opt.derivatives==FIRST_DERIV){
    printf("Gradient:\n");
    printarray(dx,nx);
  }

  if(Sip_Opt.derivatives==NO_DERIV){
    /* Central differences
    *fx=penaltylf(nx, x, nt, t, nl, lag, miu, pt, cs);
    for(i=0; i<nx;i++){
      memcpy(x1, x, nx*sizeof(double));
      memcpy(x2, x, nx*sizeof(double));
      x1[i]+=Sip_Opt.fdiff;
      x2[i]-=Sip_Opt.fdiff;
      fx1=penaltylf(nx, x1, nt, t, nl, lag, miu, pt, dummy);
      fx2=penaltylf(nx, x2, nt, t, nl, lag, miu, pt, dummy);
      dx[i]=(fx1-fx2)/(2*Sip_Opt.fdiff);
      } */
    
    /* forward differences */
    for(i=0; i<nx;i++){
      memcpy(x1, x, nx*sizeof(double));
      x1[i]+=Sip_Opt.fdiff;
      fx1=penaltylf(nx, x1, nt, t, nl, lag, miu, pt, cs1, NO_DERIV, NULL,
		    NULL);
      dx[i]=(fx1- *fx)/(Sip_Opt.fdiff);
      if(Sip_Opt.multipliersUPD==GENINV) /* need jacobian */
	for(l=0;l<nl;l++) /* Jacobian by finite differences */
	  jac[l*nx+i]=(cs1[l]-cs[l])/(Sip_Opt.fdiff);
    }
    printf("Gradient with forward differences:\n");
    printarray(dx,nx);
  }
#if !defined __linux__ && !defined linux
  free(x1);
  free(cs1);
#endif
}


void larmijo(int nx, double *x0, double *dir, double *x1,
	    double *grad, double miu, int nl, double *lag)
{
  int l;
  double alpha, dif, phi0, phiomega, nor, gd, pt;

#if defined __linux__ || defined linux
  double tsample[ntsip], dummy[nl];
#else
  double *tsample, *dummy;
  tsample=mymalloc(ntsip*sizeof(double));
  dummy=mymalloc(nl*sizeof(double));
#endif



  alpha=1;
  phi0=penaltylf(nx, x0, ntsip, tsample, nl, lag, miu, &pt, dummy,
		 NO_DERIV, NULL, NULL);
  /*  printf("phi0=%f\n", phi0);*/

  gd=0;
  nor=0;
  for(l=0;l<nx;l++){
    gd+=grad[l]*dir[l];
    nor+=dir[l]*dir[l];
  }

  nor=sqrt(nor);

/*    if(nor<1e-4){ */
/*      printf("Step=1, Maratos efect\n"); */
/*      for(l=0;l<nx;l++) */
/*        x1[l]=x0[l]+dir[l]; */
/*      return; */
/*    } */


  for(;;){
    for(l=0;l<nx;l++)
      x1[l]=x0[l]+alpha*dir[l];
    phiomega=penaltylf(nx, x1, ntsip, tsample, nl, lag, miu, &pt, dummy,
		       NO_DERIV, NULL, NULL);
    printf("alpha=%f phiomega=%f\n",alpha, phiomega);

    dif=phi0-phiomega;
    if(dif<-(Sip_Opt.armijo)*alpha*gd){
      alpha=alpha/2;
      if(alpha*nor<=1e-8){
	for(l=0;l<nx;l++)
	  x1[l]=x0[l]+alpha*dir[l];
	printf("Step zero %.20f\n", alpha);
#if !defined __linux__ && !defined linux
	free(tsample);
	free(dummy);
#endif
	return;
      }
    } else {
      printf("Step=%f\n",alpha);
#if !defined __linux__ && !defined linux
      free(tsample);
      free(dummy);
#endif
      return;
    }
  }
#if !defined __linux__ && !defined linux
  free(tsample);
  free(dummy);
#endif

}

void largo(int nx, double *x0, double *dir, double *x1,
	    double *grad, double miu, int nl, double *lag)
{
  int l;
  double alpha, dif, phi0, phiomega, nor, gd, pt;

#if defined __linux__ || defined linux
  double tsample[ntsip], dummy[nl];
#else
  double *tsample, *dummy;
  tsample=mymalloc(ntsip*sizeof(double));
  dummy=mymalloc(nl*sizeof(double));
#endif



  alpha=1;
  phi0=penaltylf(nx, x0, ntsip, tsample, nl, lag, miu, &pt, dummy,
		 NO_DERIV, NULL, NULL);
  /*  printf("phi0=%f\n", phi0);*/

  gd=0;
  nor=0;
  for(l=0;l<nx;l++){
    gd+=grad[l]*dir[l];
    nor+=dir[l]*dir[l];
  }

  nor=sqrt(nor);

  for(;;){
    for(l=0;l<nx;l++)
      x1[l]=x0[l]+alpha*dir[l];
    phiomega=penaltylf(nx, x1, ntsip, tsample, nl, lag, miu, &pt, dummy,
		       NO_DERIV, NULL, NULL);
    /*    printf("alpha=%f phiomega=%f\n",alpha, phiomega);*/

    dif=phi0-phiomega;
    if(dif<-1e-5*alpha*gd || dif>-0.9*alpha*gd){
      /*    if(dif<0){*/
      alpha=alpha/2;
      if(alpha*nor<=Sip_Opt.zero){
	alpha=0;
	for(l=0;l<nx;l++)
	  x1[l]=x0[l]+alpha*dir[l];
	printf("Step=%f, zero\n", alpha);
#if !defined __linux__ && !defined linux
	free(tsample);
	free(dummy);
#endif
	return;
      }
    } else {
      printf("Step=%f\n",alpha);
#if !defined __linux__ && !defined linux
      free(tsample);
      free(dummy);
#endif
      return;
    }
  }
#if !defined __linux__ && !defined linux
  free(tsample);
  free(dummy);
#endif

}


void lwolfe(int nx, double *x0, double *dir, double *x1, double *grad,
	    double miu, int nl, double *lag)
{
  int l;
  double alpha, dif, phi0, phiomega, nor, gd, pt, gfd;

#if defined __linux__ || defined linux
  double tsample[ntsip], dummy1[nl], dummy2[nx*nl], g1[nx];
#else
  double *tsample, *dummy1, *dummy2, *g1;
  tsample=mymalloc(ntsip*sizeof(double));
  dummy1=mymalloc(nl*sizeof(double));
  dummy2=mymalloc(nl*nx*sizeof(double));
  g1=mymalloc(nx*sizeof(double));
#endif




  alpha=1.0;
  phi0=penaltylf(nx, x0, ntsip, tsample, nl, lag, miu, &pt, dummy1,
		 NO_DERIV, NULL, NULL);
  /*  printf("phi0=%f\n", phi0);*/

  gd=0.0;
  nor=0.0;
  for(l=0;l<nx;l++){
    gd+=grad[l]*dir[l];
    nor+=dir[l]*dir[l];
  }

  nor=sqrt(nor);

  for(;;){
    for(l=0;l<nx;l++)
      x1[l]=x0[l]+alpha*dir[l];
    gpenaltylf(nx, x1, ntsip, tsample, nl, lag, miu, g1, &phiomega, &pt,
	       dummy1, dummy2);
    printf("alpha=%f phiomega=%f\n",alpha, phiomega);

    dif=phi0-phiomega;
    gfd=0.0;
    for(l=0;l<nx;l++)
      gfd+=g1[l]*dir[l];
    if(dif<-(Sip_Opt.wolfe1)*alpha*gd || gfd<(Sip_Opt.wolfe2)*gd){
      alpha=alpha/2;
      if(alpha*nor<=Sip_Opt.zero){
	for(l=0;l<nx;l++)
	  x1[l]=x0[l]+alpha*dir[l];
	printf("Step=%f, zero\n", alpha);
#if !defined __linux__ && !defined linux
	free(tsample);
	free(dummy1);
	free(dummy2);
	free(g1);
#endif
	return;
      }
    } else {
      printf("Step=%f\n",alpha);
#if !defined __linux__ && !defined linux
      free(tsample);
      free(dummy1);
      free(dummy2);
      free(g1);
#endif
      return;
    }
  }
#if !defined __linux__ && !defined linux
  free(tsample);
  free(dummy1);
  free(dummy2);
  free(g1);
#endif
}
