/*******************************************************************
Copyright (C) 2000 Universidade do Minho

umsipi = University of Minho Semi-Infinite Programming Interface

Ismael Vaz -- aivaz@dps.uminho.pt
********************************************************************/

#include <asl.h>
#include <getstub.h>
#include "sip.h"
#include "umsip.h"

/*#define asl cur_ASL*/
/*Option_Info Oinfo;*/
ASL *asl;
static fint NERROR = -1;

void printarray(real *a, int n);
void printmatrix(real *a, int n1, int n2);


void main(int argc, char **argv)
{
  char *stub, *s;
  real *X, *T, *Y, *G, *H, *HX, *HT, *HXT, *RX, *RT, *GX, *GT, *JX, *JT, *JXT;
  real fval;
  fint m, n, no, nz, mxr, mxc;
  FILE *nl;

  /* || argc>=4 || strcmp(argv[2],"-AMPL")*/
  if(argc<=1){
    usage(argc, argv);
    exit(1);
  }

  /*  printf("Called with some arguments\n");

  for(i=0;i<argc;i++)
  printf("Arg %d: %s\n", i, argv[i]);*/

  asl=ASL_alloc(ASL_read_pfgh);
  /*  stub = getstub(argv[1],&Oinfo);
  if(!stub)
  usage(argc, argv);*/

  stub=argv[1];

  want_xpi0=1;

  /*jacpdim(stub, &m, &n, &no, &nz, &mxr, &mxc, (fint)strlen(stub));*/
  /* don't call jacpdim or it will allocate X0 and you won't know if
     there is a initial guess */
  
  nl=jac_dim_ASL(asl, stub, &m, &n, &no, &nz, &mxr, &mxc, (fint)strlen(stub));
  if (!nl)
    printf("Can't read problem\n");
  
  pfgh_read_ASL(asl, nl, 0);
  
  if (mxr == 0 || mxc == 0) {
    printf("%s\n%s\n",
	   "Semi-Infinite Programming interface needs .row and .col files.",
	   "In order to provide .row and .col files, in your AMPL session say");
    if ((s = getenv("solver")))
      printf("\toption %s_auxfiles rc;\n%s", s,
	     "before saying\n\tsolve;\nor\n");
    printf("\toption auxfiles rc;\nbefore saying\n\twrite ...\n");
    exit(1);
  }
  
  want_xpi0=1;

  printf ("Existem %d variaveis\n", n_var);

  if(X0){
    printf("X0\n");
    printarray(X0,n_var);
  } else {
    printf("Sem ponto inicial\n");
  }

  sip_init(asl);

  if(X0){  
    X=REAL(nxsip); /*allocate array for X*/
    T=REAL(ntsip); /*allocate array for T*/

    sip_extractx(X0, X);
    sip_extractt(X0, T);

    printf("X=\n");
    printarray(X, nxsip);
    printf("T=\n");
    printarray(T, ntsip);
    
    Y=REAL(n_var);
    
    sip_joinxt(X, T, Y);
    printf("X+T=");
    printarray(Y, n_var);
    
    fval=sip_objval(0, X, &NERROR);
    
    printf("Objective val is %lf\n", fval);
    
    G=REAL(nxsip);
    
    sip_objgrd(0, X, G, &NERROR);
    printf("Gradiente da f. objectivo\n");
    printarray(G, nxsip);
    
    H=REAL(nxsip*nxsip);
    sip_objhes(0, H, &NERROR);
    printf("Hessiana da funcao objectivo\n");
    printmatrix(H, nxsip, nxsip);
    
    RX=REAL(nxsipc);
    RT=REAL(ntsipc);
    sip_conval(X, T, RX, RT, &NERROR);
    printf("Vector Restricoes X\n");
    printarray(RX, nxsipc);
    printf("Vector Restricoes T\n");
    printarray(RT, ntsipc);
    
    printf("Valor da primeira restricao t %lf\n",
	   sip_contval(0, X, T, &NERROR));
    
    JX=REAL(n_con*nxsip);
    JT=REAL(ntsipc*ntsip);
    JXT=REAL(ntsipc*nxsip);
    sip_jacval(X, T, JX, JT, JXT, &NERROR);
    printf("Jacobiano de X\n");
    printmatrix(JX, n_con, nxsip);
    printf("Jacobiano de T\n");
    printmatrix(JT, ntsipc, ntsip);
    
    GX=REAL(nxsip);
    GT=REAL(ntsip);
    sip_contgrd(0, X, T, GX, GT, &NERROR);
    printf("Gradiente da primeira restricao t componente x\n");
    printarray(GX, nxsip);
    printf("Gradiente da primeira restricao t componente t\n");
    printarray(GT, ntsip);
    
    sip_conxhes(0, H, &NERROR);
    printf("Hessiana da primeira restricao X\n");
    printmatrix(H, nxsip, nxsip);
    
    HX=REAL(nxsip*nxsip);
    HT=REAL(ntsip*ntsip);
    HXT=REAL(nxsip*ntsip);
    sip_conthes(0, HX, HT, HXT, &NERROR);
    printf("Hessiana da primeira restricao t em ordem a X\n");
    printmatrix(HX, nxsip, nxsip);
    printf("Hessiana da primeira restricao t em ordem a T\n");
    printmatrix(HT, ntsip, ntsip);
    
    
    solver();

    /* lets write a solution */
    write_sol("End\n", X0, NULL, NULL);
    sip_free();
    /* so ampl doesn't complains */
    exit(0);
  } else {
    sip_free();
    /* return to ampl with no solution */
    exit(1);
  }
}


void usage(int argc, char** argv)
{
  printf("Usage: %s stub -AMPL\n",argv[0]);
}


void printarray(real *a,int n)
{
  int i;

  for(i=0;i<n;i++)
    printf("%lf ", a[i]);
  printf("\n");
}

void printmatrix(real *m, int n1, int n2)
{
  int i;

  for(i=0;i<n1;i++)
    printarray(m+n2*i, n2);
}
