/*******************************************************************

sip.h = Semi-Infinite Programming

Some definitions for Semi-Infinite Programming using ampl

Ismael Vaz -- aivaz@dps.uminho.pt
********************************************************************/

#ifndef SIP_included
#define SIP_included

#define DOUBLE(n) Malloc(n*sizeof(long))
#define REAL(n) Malloc(n*sizeof(real))

enum {
  SIP_OK=0,
  SIP_invalid=1
};

typedef struct SIP {
  int tvars;         /* number of t variables */
  int *tvars_i;      /* array with t variables distance*/
  int xvars;         /* number of x variables */
  int *xvars_i;      /* array with x variables distance*/
  int nxcon;         /* number of finite constraints */
  int ntcon;         /* number of infinite constraints */
  int *tcon_i;       /* array with t constraints distance */
  int *xcon_i;       /* array with x constraints distance */
  real *xbl, *xbu;   /* array with x vars lower and upper bounds */
  real *tbl, *tbu;   /* array with t vars lower and upper bounds */
  real *xcbl, *xcbu; /* array with x constraints lower and upper bounds */
  real *tcbl, *tcbu; /* array with t constraints lower and upper bounds */
} SIP;

SIP sip;

extern int sip_init(ASL *a);
extern void sip_extractx(real *a, real *b);
extern void sip_extractt(real *a, real *b);
extern void sip_joinxt(real *a, real *b, real *c);
extern real sip_objval(int nobj, real *X, fint *nerror);
extern void sip_objgrd(int nobj, real *X, real *G, fint *nerror);
extern void sip_objhes(int nobj, real *H, fint *nerror);
extern void sip_conval(real *X, real *T, real *RX, real *RT, fint *nerror);
extern void sip_jacval(real *X, real *T, real *JX, real *JT, real *JXT, fint *nerror);
extern real sip_conxval(int ncon, real *X, fint *nerror);
extern real sip_contval(int ncon, real *X, real *T, fint *nerror);
extern void sip_conxgrd(int ncon, real *X, real *G, fint *nerror);
extern void sip_contgrd(int ncon, real *X, real *T, real *GX, real *GT, fint *nerror);
extern void sip_conxhes(int ncon, real *H, fint *nerror);
/*  extern void sip_conthes(int ncon, real *HX, real *HT, fint *nerror); */
extern void sip_conthes(int ncon, real *HX, real *HT, real *HXT, fint *nerror);
extern void sip_free(void);

#define nxsip  sip.xvars
#define xsipd  sip.xvars_i
#define ntsip  sip.tvars
#define tsipd  sip.tvars_i
#define nxsipc sip.nxcon
#define ntsipc sip.ntcon
#define xsipcd sip.xcon_i
#define tsipcd sip.tcon_i
#define XBU    sip.xbu
#define XBL    sip.xbl
#define TBU    sip.tbu
#define TBL    sip.tbl
#define XCBU   sip.xcbu
#define XCBL   sip.xcbl
#define TCBU   sip.tcbu
#define TCBL   sip.tcbl


#endif SIP_included
