###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#
#   As described by Shu-Cherng Fang and Soon-Yi Wu
#	in "An Inexact Approach to Solving Linear Semi-Infinite
#		  Programming Problems"
#    Optimization, vol. 28, pp. 291-299, 1994
#
#   Problem 2
#    Coded by A. Ismael F. Vaz 27/02/2000  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of x vars
param nx;
#number of t
param nt;

# initial guess
param xinit{1..nx};
param tinit{1..nt};

# x variables
var x {i in 1..nx} := xinit[i];
# t variable
var t {j in 1..nt} := tinit[j];

# pay attention, in article this problem is a maximization
minimize fx:
	(sum {i in 1..nx} (x[i]/i));

subject to tcons:
	(sum {i in 1..nx} (-t[1]^(i-1)*x[i]))+(1/(2-t[1])) <= 0;

subject to bounds:
	0 <= t[1] <= 1;

subject to xbounds {i in 1..nx}:
	x[i] >= 0;

data;

param nx := 50;
param nt := 1;

# inital guess
param xinit:=
1 1	2 1	3 1	4 1	5 1	6 1	7 1	8 1	9 1	10 1
11 1	12 1	13 1	14 1	15 1	16 1	17 1	18 1	19 1	20 1
21 1	22 1	23 1	24 1	25 1	26 1	27 1	28 1	29 1	30 1
31 1	32 1	33 1	34 1	35 1	36 1	37 1	38 1	39 1	40 1
41 1	42 1	43 1	44 1	45 1	46 1	47 1	48 1	49 1	50 1;
param tinit:=
	1 0;

# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "f=%lf\n", 0.69314805;
