###############################################################################
# Objective: Linear
# Constraints: Generic
###############################################################################
#
#   As described by Stephan Goerner in "Ein Hybridverfahren zur Loesung
#                         nichtlinearer semi-infiniter Optimierungsprobleme",
#   PhD. Dissertation, Berlin University, 1997
#
#   Example 18
#    Coded by A. Ismael F. Vaz 16/01/2002  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

# parameter
param n;
# number of x vars
param nx;
#number of t
param nt;

# starting point (ampl needs also a starting point for t)
param xinit{1..nx};
param tinit{1..nt};

# x variables
var x {i in 1..nx} := xinit[i];
# t variable
var t {j in 1..nt} := tinit[j];

minimize fx:
	x[nx];

subject to tcons1:
	1/(1+t[1])-(sum {i in 1..n} (x[i]*exp(x[n+i]*t[1])))-x[nx] <= 0;
subject to tcons2:
	-1/(1+t[1])+(sum {i in 1..n} (x[i]*exp(x[n+i]*t[1])))-x[nx] <= 0;
subject to bounds:
	-0.5 <= t[1] <= 0.5;

data;

#tested for n=1, 2, 3
#param n  := 1; 
#param nx := 3;
#param n  := 2; 
#param nx := 5;
param n  := 3; 
param nx := 7;
param nt := 1;

# Goerner starting points
#param xinit :=
#	1 1
#	2 -1
#	3 1;
#param xinit :=
#	1 1
#	2 1
#	3 -3
#	4 -1
#	5 1;
param xinit :=
	1 1
	2 1
	3 1
	4 -7
	5 -3
	6 -1	
	7 1;


param tinit :=
	1 0;

# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
#printf "x=(%lf,%lf,%lf) f=%lf\n", 1.052855, -1.194167, 0.087160, 0.087160;
#printf "x=(%lf,%lf,%lf,%lf,%lf) f=%lf\n", 0.098080, 0.901648, -4.070259,
#	 -0.649030, 0.002070, 0.002070;
printf "x=(%lf,%lf,%lf,%lf,%lf,%lf,%lf) f=%lf\n", 0.003955, 0.243536,
	 0.752491, -7.521090, -2.581921, -0.4544, 0.000044, 0.000044;
