###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#
#   As described by Stephan Goerner in "Ein Hybridverfahren zur Loesung
#                         nichtlinearer semi-infiniter Optimierungsprobleme",
#   PhD. Dissertation, Berlin University, 1997
#
#   Example 23
#    Coded by A. Ismael F. Vaz 16/01/2002  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of x vars
param nx;
#number of t
param nt;

# starting point (ampl needs also a starting point for t)
param xinit{1..nx};
param tinit{1..nt};

# x variables
var x {i in 1..nx} := xinit[i];
# t variable
var t {j in 1..nt} := tinit[j];

minimize fx:
	x[16];

subject to tcons1:
	exp(t[2]*log(t[1]))-(x[1]+x[2]*t[2]+x[3]*t[2]^2+x[4]*t[2]^3+x[5]*t[2]^4+t[1]*(x[6]+x[7]*t[2]+x[8]*t[2]^2+x[9]*t[2]^3)+t[1]^2*(x[10]+x[11]*t[2]+x[12]*t[2]^2)+t[1]^3*(x[13]+x[14]*t[2])+x[15]*t[1]^4)-x[16] <=0;
subject to tcons2:
	-exp(t[2]*log(t[1]))+(x[1]+x[2]*t[2]+x[3]*t[2]^2+x[4]*t[2]^3+x[5]*t[2]^4+t[1]*(x[6]+x[7]*t[2]+x[8]*t[2]^2+x[9]*t[2]^3)+t[1]^2*(x[10]+x[11]*t[2]+x[12]*t[2]^2)+t[1]^3*(x[13]+x[14]*t[2])+x[15]*t[1]^4)-x[16] <=0;
subject to bounds {i in 1..nt}:
	1 <= t[i] <= 2;

data;

param nx := 16;
param nt := 2;

# watson starting point
param xinit :=
	1 2
	2 -4
	3 2
	4 -0.1
	5 0.1
	6 -2
	7 5
	8 -2
	9 0.2
	10 1
	11 -0.1
	12 1
	13 -0.1
	14 -0.1
	15 0.1
	16 0.1;

param tinit :=
	1 0
	2 0;

# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "x=(%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf) f=%lf\n", 2.402567, -2.913281, 1.797893, -0.226116, 0.006187, -1.887523, 4.942918,
	-2.346462, 0.156936, 0.625585, -1.316046, 0.775617, -0.012086,
	 -0.004980, -0.000130, 0.001077, 0.001077;
