###############################################################################
# Objective: Linear
# Constraints: Generic
###############################################################################
#
#   As described by Stephan Goerner in "Ein Hybridverfahren zur Loesung
#                         nichtlinearer semi-infiniter Optimierungsprobleme",
#   PhD. Dissertation, Berlin University, 1997
#
#   Example 24
#    Coded by A. Ismael F. Vaz 16/01/2002  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of x vars
param nx;
#number of t
param nt;

# starting point (ampl needs also a starting point for t)
param xinit{1..nx};
param tinit{1..nt};

# x variables
var x {i in 1..nx} := xinit[i];
# t variable
var t {j in 1..nt} := tinit[j];

minimize fx:
	x[8];

subject to tcons1:
	sin(t[1])*sin(t[2])-((2*x[1]+x[2]*(t[1]+t[2])+x[3]*(t[1]^2+t[2]^2)+x[4]*(t[1]^3+t[2]^3))/(2+x[5]*(t[1]+t[2])+x[6]*(t[1]^2+t[2]^2)+x[7]*(t[1]^3+t[2]^3)))-x[8] <= 0;
subject to tcons2:
	-sin(t[1])*sin(t[2])+((2*x[1]+x[2]*(t[1]+t[2])+x[3]*(t[1]^2+t[2]^2)+x[4]*(t[1]^3+t[2]^3))/(2+x[5]*(t[1]+t[2])+x[6]*(t[1]^2+t[2]^2)+x[7]*(t[1]^3+t[2]^3)))-x[8] <= 0;
subject to bounds {i in 1..nt}:
	0 <= t[i] <= 1;

data;

param nx := 8;
param nt := 2;

# watson starting point
param xinit :=
	1 0
	2 0.1
	3 -0.3
	4 0.8
	5 -2
	6 2
	7 -0.8
	8 0.23;

param tinit :=
	1 0
	2 0;

# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "x=(%lf,%lf,%lf,%lf,%lf,%lf,%lf,%lf) f=%lf\n", -0.30634, 0.439511,
	-0.709553, 0.364594, -2.672893, 2.696643, -0.937220, 0.030634, 0.030634;
