###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#  Fractional semi-infinite programming problem
#  As described by Martin Gugat, PhD Thesis,
#   submitted to Trier University, "Fractional Semi-Infinite Programming", 1994
#
#   Stochastic data example
#    Coded by A. Ismael F. Vaz 19/03/2002  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of z vars
param nz := 6;

# Gamma parameter
param gamma := 10^4;

# stochastic data
param data {0..39};

# z variables
var z {1..nz};
# infinite variable
var t;
# d distance
var d;


# The functions
var v = z[3]*t^2+z[2]*t+z[1];
var w = z[6]*t^2+z[5]*t+z[4];



minimize dp:
	d;

subject to cons1:
	-(sum {i in 0..39} (data[i]-
	(z[3]*(i*0.5)^2+z[2]*(i*0.5)+z[1])/
	(z[6]*(i*0.5)^2+z[5]*(i*0.5)+z[4])))-d <= 0;
subject to cons2:
	(sum {i in 0..39} (data[i]-
	(z[3]*(i*0.5)^2+z[2]*(i*0.5)+z[1])/
	(z[6]*(i*0.5)^2+z[5]*(i*0.5)+z[4])))-d <= 0;
subject to tcons3:
	-w <= -1;
subject to tcons4:
	w <= gamma;
subject to bounds:
	0 <= t <= 19.5; # I guess !!!!


###################
# End of Problem  #
###################

data;

param data :=
	0	0.249
	1	0.238
	2	0.202
	3	0.067
	4	-0.123

	5	0.029
	6	0.158
	7	0.172
	8	0.189
	9	0.168

	10	0.067
	11	0.103
	12	0.107
	13	0.075
	14	0.049

	15	0.044
	16	0.030
	17	0.028
	18	0.048
	19	0.071

	20	0.080
	21	0.067
	22	0.087
	23	0.040
	24	0.047

	25	0.047
	26	0.064
	27	0.068
	28	0.081
	29	0.062

	30	0.055
	31	0.063
	32	0.059
	33	0.076
	34	0.075

	35	0.080
	36	0.106
	37	0.113
	38	0.098
	39	0.072;

# don't forget to write .col and .row files
option nsips_auxfiles rc;
# this problem have no initial guess (starting point)
option reset_initial_guesses 1;
# change solver
option solver nsips;
# solve problem
solve;


##################
# Solution found #
##################
printf "Solution found\n";
display d;
display z;
