###############################################################################
# Objective: Linear
# Constraints: Quadratic
###############################################################################
#
#  This problems is hettich10, corrected.
#  It is also described by  J.L. Zhou and A.L. Tits, in
#      "An SQP algorithm for finely discretized continuous minimax problems
#                  and other minimax problems with many objective functions"
#    SIAM Journal on Optimization, vol. 6, no. 2, pp. 461-487, 1996
#   Example HET-Z
#
#    Coded by A. Ismael F. Vaz 03/06/2002  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

#number of t
param nt := 1;
#number of x
param nx := 2;

# initial guess
param tinit{1..nt};
param xinit{1..nx};

# t variable
var t {i in 1..nt} := tinit[i];
# x variables
var x {i in 1..nx} := xinit[i];

minimize fx:
	x[2];

subject to tcons1:
	-(1-t[1]^2)+(0.5*x[1]^2-2*x[1]*t[1])-x[2] <=0;
subject to tcons2:
	 (1-t[1]^2)-(0.5*x[1]^2-2*x[1]*t[1])-x[2] <=0;
subject to bounds {i in 1..nt}:
	-1 <= t[i] <= 1;


###################
# End of Problem  #
###################

data;

param xinit :=
	1	1
	2	1;

param tinit :=
	1	0;

# don't forget to write .col and .row files
option nsips_auxfiles rc;
# change solver
option solver nsips;
# solve problem
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "obj=%lf\n", 0.99995000;
