###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#
#  Hettich problem where the constraint is the unit disk
#
#  As described by R. Hettich in "A review of numerical methods for
#		semi-infinite optimization"
#   in A.V. Fiacco and K.O. Kortanek (eds.), "Semi-infinite programming and
#		and applications",
#    Lecture notes in Economics and Mathematical Systems no.215, pp. 158-178
#    Springer Verlag, Berlin (1981)
#
#   Example in page 165
#    Coded by A. Ismael F. Vaz 08/12/2003  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

#number of t
param nt := 2;
#number of x
param nx := 2;

# t variable
var t {1..nt};
# x variables
var x {1..nx};

minimize fx:
	x[1]+2*x[2];

subject to tcons:
	2*t[1]*x[1]+2*t[2]*x[2]-(t[1]^2+t[2]^2+1)<=0;
subject to bounds {i in 1..nt}:
	-2 <= t[i] <= 2;


###################
# End of Problem  #
###################

# don't forget to write .col and .row files
option nsips_auxfiles rc;
# change solver
option solver nsips;
# solve problem
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;
