###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#  As described by W. van Honstede "An approximation method for semi-infinite
#                 problems",
#  In R. Hettich (eds.), "Semi-infinite Programming",
#    Lecture notes in Control and Information Sciences no. 15, pp. 126-136
#    Springer Verlag, Berlin (1979)
#
#
#   Example 3 (Example 1 and 2 in watson10.mod...watson13.mod)
#
#    Coded by A. Ismael F. Vaz 11/12/2003  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of x vars
param nx;
#number of t
param nt;

# starting point (ampl needs also a starting point for t)
param xinit{1..nx};
param tinit{1..nt};

# x variables
var x {i in 1..nx} := xinit[i];
# t variable
var t {j in 1..nt} := tinit[j];

minimize fx:
	2*x[1]+4*x[2]+x[3];

subject to tcons:
	(1-x[1])*(if(t[1]>0) then ((2/t[1])*exp(-1*(1+(t[2]-1)^2)/t[1]))
			     else 0)+
	(1-x[2])*(if(t[1]>0) then ((1/(2*t[1]))*exp(-1*(4+t[2]^2)/(4*t[1])))
			     else 0)+
	(1-x[3])*(if(t[1]>2) then ((1/(t[1]-2))*exp(-1*(1+(t[2]+1)^2)/(t[1]-2)))
			     else 0)-
	0.5 <= 0;
	
subject to xbounds {i in 1..nx}:
	0 <= x[i] <= 1;
subject to tbounds {i in 1..nt}:
	-1 <= t[i] <= 4;

data;

param nx := 3;
param nt := 2;

# watson starting point
param xinit :=
	1 0
	2 0
	3 1;

param tinit :=
	1 0
	2 0;

# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "x=(%lf,%lf,%lf) f=%lf\n", 0.54315, 0, 0.12615, 1.21244;
