###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#
#  SIP problem presented by Powell to show that his Karmarkar algorithm can
#  converge to a non optimal point.
#   As described by M.L. Todd , "Interior-point algorithms for semi-infinite
#                                 programming",
#   in Mathematical Programming, vol. 65, 1994, pp. 217-245
#
#
#    Coded by A. Ismael F. Vaz 20/11/2001  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

# pi
param pi := 4*atan(1);

#number of variables
param nx := 2;

#number of t
param nt := 1;

# x variables
var x {1..nx};
# t variable
var t {1..nt};


minimize fx:
	x[2];

subject to tcons:
	cos(2*pi*t[1])*x[1]+sin(2*pi*t[1])*x[2]-1 <= 0;
subject to bound:
	0 <= t[1] <= 1;

data;

let x[1] := 0.599;
let x[2] := 0.8;

###################
# End of Problem  #
###################

# don't forget to write .col and .row files
option nsips_auxfiles rc;
# change solver
option solver nsips;
# solve problem
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;


################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "x=(%lf, %lf)\n", 0, -1;
