###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#
#  Reemtsen set of SIP problems (Chebyshev approximation problems)
#   As described by R. Reemtsen, "Discretization Methods for the Solution of
#                                 Semi-Infinite Programming Problems",
#   in Journal of Optimization Theory and Applications, vol. 71, No. 1, 1991
#
#   Example 4.6
#    Coded by A. Ismael F. Vaz 31/12/1999  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

#number of t vars
param nt := 2;

# t parameter, t=2..5 tested by Reemtsen
param pt := 5;

# x variables
var x {0..pt,0..pt};
# t variable
var t {1..nt};
# distance variable
var dist;

minimize distance:
	dist;

subject to tcons1:
	-(sum {i in 0..pt} ( sum {j in 0..pt} (x[i,j]*t[1]^(i)*t[2]^j))
          -(sqrt(t[1]+2*t[2]+4)))-dist <= 0;
subject to tcons2:
	(sum {i in 0..pt} ( sum {j in 0..pt} (x[i,j]*t[1]^(i)*t[2]^j))
          -(sqrt(t[1]+2*t[2]+4)))-dist <= 0;
subject to bound1:
	-1 <= t[1] <= 1;
subject to bound2:
	-1 <= t[2] <= 1;


###################
# End of Problem  #
###################

# don't forget to write .col and .row files
option nsips_auxfiles rc;
# this problem have no initial guess (starting point)
option reset_initial_guesses 1;
# change solver
option solver nsips;
# solve problem
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display distance;


################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "distance=%lf\n", 1.140057e-2;
