###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#
#  SIP problem presented by G. Still to illustrate the need to include bound
#  points in the discretization method to obtain a better convergence
#   As described by G. Still, "Discretization in semi-infinite programming:
#                                 the rate of convergence",
#   in Mathematical Programming, vol. 91, 2001, pp. 53-69
#
#
#    Coded by A. Ismael F. Vaz 27/11/2001  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

#number of variables
param nx := 2;

#number of t
param nt := 1;

# x variables
var x {1..nx};
# t variable
var t {1..nt};


minimize fx:
	-x[1]+x[2];

subject to tcons:
	x[1]*(1-t[1])*(1+t[1])+t[1]^2*(t[1]^2-x[2]) <= 0;
subject to bound:
	-1 <= t[1] <= 1;


###################
# End of Problem  #
###################

# no initial guess provided
option reset_initial_guesses 1;
# don't forget to write .col and .row files
option nsips_auxfiles rc;
# change solver
option solver nsips;
# solve problem
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;


################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "x=(%lf, %lf)\n", 0, 1;
