###############################################################################
# Objective: Polynomial
# Constraints: Linear
###############################################################################
#  Watson set of SIP problems
#   As described by G.A. Watson in "Numerical Experiments with Globally
#                 Convergent Methods for Semi-Infinite Programming Problems",
#   Edited in Fiacco, A.V., and K.O. Kortanek (eds.),
#    "Semi-infinite Programming and Applications",
#    Lecture notes in Economics and Mathematical Systems no. 215, pp.193-205
#    Springer Verlag, Berlin (1983)
#
#    Original from W. van Honstede (Example 2, without finite bounds)
#
#   Example 13
#    Coded by A. Ismael F. Vaz 24/12/1999  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

# number of x vars
param nx;
#number of t
param nt;

# starting point (ampl needs also a starting point for t)
param xinit{1..nx};
param tinit{1..nt};

# x variables
var x {i in 1..nx} := xinit[i];
# t variable
var t {j in 1..nt} := tinit[j];

minimize fx:
	2*x[1]+4*x[2]+x[3]+30*x[3]^2+30*x[3]^3;

subject to tcons:
	(1-x[1])*(if(t[1]>0) then ((1/t[1])*exp(-1*(1+(t[2]-1)^2)/t[1]))
			     else 0)+
	(1-x[2])*(if(t[1]>0) then ((1/t[1])*exp(-1*(8+t[2]^2)/(4*t[1])))
			     else 0)+
	(1-x[3])*(if(t[1]>2) then ((1/(t[1]-2))*exp(-1*(1+(t[2]+1)^2)/(t[1]-2)))
			     else 0)-
	0.5 <= 0;
	
subject to tbounds {i in 1..nt}:
	-1 <= t[i] <= 4;

data;

param nx := 3;
param nt := 2;

# watson starting point
param xinit :=
	1 0
	2 0
	3 0;

param tinit :=
	1 0
	2 0;

# solve problem
# don't forget to write .col and .row files
option nsips_auxfiles rc;
option solver nsips;
solve;

##################
# Solution found #
##################
printf "Solution found\n";
display x;
display fx;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "x=(%lf,%lf,%lf) f=%lf\n", -0.063763, 0.388230, 0.111354, 1.950168;
