###############################################################################
# Objective: Linear
# Constraints: Linear
###############################################################################
#
#  As described by J.L. Zhou and A.L. Tits
#      "An SQP algorithm for finely discretized continuous minimax problems
#                  and other minimax problems with many objective functions"
#    SIAM Journal on Optimization, vol. 6, no. 2, pp. 461-487, 1996
#
#   Example PT
#
#    Coded by A. Ismael F. Vaz 03/06/2002  aivaz@dps.uminho.pt
#    University of Minho, Portugal
###############################################################################

#number of x vars
param nx;
#number of t vars
param nt;
# initial guess
param x0 {0..nx-1};
param t0 {1..nt};

# x variables
var x {i in 0..nx-1} := x0[i];
# t variable
var t {i in 1..nt} := t0[i];

minimize obj:
	x[0];

subject to tcons1:
	(2*t[1]^2-1)*x[1]+t[1]*(1-t[1])*(1-x[1])-x[0] <=0;
subject to bounds {i in 1..nt}:
	0 <= t[i] <= 1;


###################
# End of Problem  #
###################

data;

param nx := 2;
param nt := 1;

param x0 :=
	0	1
	1	5;

param t0 :=
	1	0;

# don't forget to write .col and .row files
option nsips_auxfiles rc;
# change solver
option solver nsips;
# solve problem
solve;


##################
# Solution found #
##################
printf "Solution found\n";
display x;
display obj;

################################################
# solution of problem presented by the authors #
################################################
printf "Original Solution\n";
printf "obj=%lf\n", 0.23605381;
